@@ -1,13 +1,13 @@
 package stats
 
 import (
-	"fmt"
-	"log"
 	"math"
 	"runtime"
 	"sync"
 	"time"
 
+	hclog "github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/nomad/plugins/device"
 	"github.com/shirou/gopsutil/cpu"
 	"github.com/shirou/gopsutil/disk"
 	"github.com/shirou/gopsutil/host"
@@ -20,6 +20,7 @@ type HostStats struct {
 	CPU              []*CPUStats
 	DiskStats        []*DiskStats
 	AllocDirStats    *DiskStats
+	DeviceStats      []*DeviceGroupStats
 	Uptime           uint64
 	Timestamp        int64
 	CPUTicksConsumed float64
@@ -53,6 +54,12 @@ type DiskStats struct {
 	InodesUsedPercent float64
 }
 
+// DeviceGroupStats represents stats related to device group
+type DeviceGroupStats = device.DeviceGroupStats
+
+// DeviceStatsCollector is used to retrieve all the latest statistics for all devices.
+type DeviceStatsCollector func() []*DeviceGroupStats
+
 // NodeStatsCollector is an interface which is used for the purposes of mocking
 // the HostStatsCollector in the tests
 type NodeStatsCollector interface {
@@ -62,30 +69,34 @@ type NodeStatsCollector interface {
 
 // HostStatsCollector collects host resource usage stats
 type HostStatsCollector struct {
-	numCores        int
-	statsCalculator map[string]*HostCpuStatsCalculator
-	logger          *log.Logger
-	hostStats       *HostStats
-	hostStatsLock   sync.RWMutex
-	allocDir        string
+	numCores             int
+	statsCalculator      map[string]*HostCpuStatsCalculator
+	hostStats            *HostStats
+	hostStatsLock        sync.RWMutex
+	allocDir             string
+	deviceStatsCollector DeviceStatsCollector
 
 	// badParts is a set of partitions whose usage cannot be read; used to
 	// squelch logspam.
 	badParts map[string]struct{}
+
+	logger hclog.Logger
 }
 
 // NewHostStatsCollector returns a HostStatsCollector. The allocDir is passed in
 // so that we can present the disk related statistics for the mountpoint where
 // the allocation directory lives
-func NewHostStatsCollector(logger *log.Logger, allocDir string) *HostStatsCollector {
+func NewHostStatsCollector(logger hclog.Logger, allocDir string, deviceStatsCollector DeviceStatsCollector) *HostStatsCollector {
+	logger = logger.Named("host_stats")
 	numCores := runtime.NumCPU()
 	statsCalculator := make(map[string]*HostCpuStatsCalculator)
 	collector := &HostStatsCollector{
-		statsCalculator: statsCalculator,
-		numCores:        numCores,
-		logger:          logger,
-		allocDir:        allocDir,
-		badParts:        make(map[string]struct{}),
+		statsCalculator:      statsCalculator,
+		numCores:             numCores,
+		logger:               logger,
+		allocDir:             allocDir,
+		badParts:             make(map[string]struct{}),
+		deviceStatsCollector: deviceStatsCollector,
 	}
 	return collector
 }
@@ -105,21 +116,25 @@ func (h *HostStatsCollector) collectLocked() error {
 	// Determine up-time
 	uptime, err := host.Uptime()
 	if err != nil {
-		return err
+		h.logger.Error("failed to collect upstime stats", "error", err)
+		uptime = 0
 	}
 	hs.Uptime = uptime
 
 	// Collect memory stats
 	mstats, err := h.collectMemoryStats()
 	if err != nil {
-		return err
+		h.logger.Error("failed to collect memory stats", "error", err)
+		mstats = &MemoryStats{}
 	}
 	hs.Memory = mstats
 
 	// Collect cpu stats
 	cpus, ticks, err := h.collectCPUStats()
 	if err != nil {
-		return err
+		h.logger.Error("failed to collect cpu stats", "error", err)
+		cpus = []*CPUStats{}
+		ticks = 0
 	}
 	hs.CPU = cpus
 	hs.CPUTicksConsumed = ticks
@@ -127,16 +142,22 @@ func (h *HostStatsCollector) collectLocked() error {
 	// Collect disk stats
 	diskStats, err := h.collectDiskStats()
 	if err != nil {
-		return err
+		h.logger.Error("failed to collect disk stats", "error", err)
+		hs.DiskStats = []*DiskStats{}
 	}
 	hs.DiskStats = diskStats
 
 	// Getting the disk stats for the allocation directory
 	usage, err := disk.Usage(h.allocDir)
 	if err != nil {
-		return fmt.Errorf("failed to find disk usage of alloc_dir %q: %v", h.allocDir, err)
+		h.logger.Error("failed to find disk usage of alloc", "alloc_dir", h.allocDir, "error", err)
+		hs.AllocDirStats = &DiskStats{}
+	} else {
+		hs.AllocDirStats = h.toDiskStats(usage, nil)
 	}
-	hs.AllocDirStats = h.toDiskStats(usage, nil)
+	// Collect devices stats
+	deviceStats := h.collectDeviceGroupStats()
+	hs.DeviceStats = deviceStats
 
 	// Update the collected status object.
 	h.hostStats = hs
@@ -175,7 +196,7 @@ func (h *HostStatsCollector) collectDiskStats() ([]*DiskStats, error) {
 			}
 
 			h.badParts[partition.Mountpoint] = struct{}{}
-			h.logger.Printf("[WARN] client: error fetching host disk usage stats for %v: %v", partition.Mountpoint, err)
+			h.logger.Warn("error fetching host disk usage stats", "error", err, "partition", partition.Mountpoint)
 			continue
 		}
 		delete(h.badParts, partition.Mountpoint)
@@ -187,6 +208,14 @@ func (h *HostStatsCollector) collectDiskStats() ([]*DiskStats, error) {
 	return diskStats, nil
 }
 
+func (h *HostStatsCollector) collectDeviceGroupStats() []*DeviceGroupStats {
+	if h.deviceStatsCollector == nil {
+		return []*DeviceGroupStats{}
+	}
+
+	return h.deviceStatsCollector()
+}
+
 // Stats returns the host stats that has been collected
 func (h *HostStatsCollector) Stats() *HostStats {
 	h.hostStatsLock.RLock()
@@ -194,7 +223,7 @@ func (h *HostStatsCollector) Stats() *HostStats {
 
 	if h.hostStats == nil {
 		if err := h.collectLocked(); err != nil {
-			h.logger.Printf("[WARN] client: error fetching host resource usage stats: %v", err)
+			h.logger.Warn("error fetching host resource usage stats", "error", err)
 		}
 	}
 
