@@ -176,6 +176,68 @@ func TestAgent_ServerConfig(t *testing.T) {
 	require.Equal(t, int32(3), out.BootstrapExpect)
 }
 
+func TestAgent_ServerConfig_SchedulerFlags(t *testing.T) {
+	cases := []struct {
+		name     string
+		input    *structs.SchedulerConfiguration
+		expected structs.SchedulerConfiguration
+	}{
+		{
+			"default case",
+			nil,
+			structs.SchedulerConfiguration{
+				PreemptionConfig: structs.PreemptionConfig{
+					SystemSchedulerEnabled: true,
+				},
+			},
+		},
+		{
+			"empty value: preemption is disabled",
+			&structs.SchedulerConfiguration{},
+			structs.SchedulerConfiguration{
+				PreemptionConfig: structs.PreemptionConfig{
+					SystemSchedulerEnabled: false,
+				},
+			},
+		},
+		{
+			"all explicitly set",
+			&structs.SchedulerConfiguration{
+				PreemptionConfig: structs.PreemptionConfig{
+					SystemSchedulerEnabled:  true,
+					BatchSchedulerEnabled:   true,
+					ServiceSchedulerEnabled: true,
+				},
+			},
+			structs.SchedulerConfiguration{
+				PreemptionConfig: structs.PreemptionConfig{
+					SystemSchedulerEnabled:  true,
+					BatchSchedulerEnabled:   true,
+					ServiceSchedulerEnabled: true,
+				},
+			},
+		},
+	}
+
+	for _, c := range cases {
+		t.Run(c.name, func(t *testing.T) {
+			conf := DefaultConfig()
+			conf.Server.DefaultSchedulerConfig = c.input
+
+			a := &Agent{config: conf}
+			conf.AdvertiseAddrs.Serf = "127.0.0.1:4000"
+			conf.AdvertiseAddrs.RPC = "127.0.0.1:4001"
+			conf.AdvertiseAddrs.HTTP = "10.10.11.1:4005"
+			conf.ACL.Enabled = true
+			require.NoError(t, conf.normalizeAddrs())
+
+			out, err := a.serverConfig()
+			require.NoError(t, err)
+			require.Equal(t, c.expected, out.DefaultSchedulerConfig)
+		})
+	}
+}
+
 // TestAgent_ServerConfig_Limits_Errors asserts invalid Limits configurations
 // cause errors. This is the server-only (RPC) counterpart to
 // TestHTTPServer_Limits_Error.
@@ -301,6 +363,50 @@ func TestAgent_ServerConfig_Limits_OK(t *testing.T) {
 	}
 }
 
+func TestAgent_ClientConfig(t *testing.T) {
+	t.Parallel()
+	conf := DefaultConfig()
+	conf.Client.Enabled = true
+
+	// For Clients HTTP and RPC must be set (Serf can be skipped)
+	conf.Addresses.HTTP = "169.254.0.1"
+	conf.Addresses.RPC = "169.254.0.1"
+	conf.Ports.HTTP = 5678
+	a := &Agent{config: conf}
+
+	if err := conf.normalizeAddrs(); err != nil {
+		t.Fatalf("error normalizing config: %v", err)
+	}
+	c, err := a.clientConfig()
+	if err != nil {
+		t.Fatalf("got err: %v", err)
+	}
+
+	expectedHttpAddr := "169.254.0.1:5678"
+	if c.Node.HTTPAddr != expectedHttpAddr {
+		t.Fatalf("Expected http addr: %v, got: %v", expectedHttpAddr, c.Node.HTTPAddr)
+	}
+
+	conf = DefaultConfig()
+	conf.DevMode = true
+	a = &Agent{config: conf}
+	conf.Client.Enabled = true
+	conf.Addresses.HTTP = "169.254.0.1"
+
+	if err := conf.normalizeAddrs(); err != nil {
+		t.Fatalf("error normalizing config: %v", err)
+	}
+	c, err = a.clientConfig()
+	if err != nil {
+		t.Fatalf("got err: %v", err)
+	}
+
+	expectedHttpAddr = "169.254.0.1:4646"
+	if c.Node.HTTPAddr != expectedHttpAddr {
+		t.Fatalf("Expected http addr: %v, got: %v", expectedHttpAddr, c.Node.HTTPAddr)
+	}
+}
+
 // Clients should inherit telemetry configuration
 func TestAgent_Client_TelemetryConfiguration(t *testing.T) {
 	assert := assert.New(t)
