@@ -4,19 +4,18 @@ import (
 	"fmt"
 	"sync"
 
-	log "github.com/hashicorp/go-hclog"
-
 	"github.com/hashicorp/consul/api"
+	"github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/nomad/helper"
 )
 
 // MockCatalog can be used for testing where the CatalogAPI is needed.
 type MockCatalog struct {
-	logger log.Logger
+	logger hclog.Logger
 }
 
-func NewMockCatalog(l log.Logger) *MockCatalog {
-	l = l.Named("mock_consul")
-	return &MockCatalog{logger: l}
+func NewMockCatalog(l hclog.Logger) *MockCatalog {
+	return &MockCatalog{logger: l.Named("mock_consul")}
 }
 
 func (m *MockCatalog) Datacenters() ([]string, error) {
@@ -111,6 +110,7 @@ func (c *MockAgent) Services() (map[string]*api.AgentService, error) {
 			ID:                v.ID,
 			Service:           v.Name,
 			Tags:              make([]string, len(v.Tags)),
+			Meta:              helper.CopyMapStringString(v.Meta),
 			Port:              v.Port,
 			Address:           v.Address,
 			EnableTagOverride: v.EnableTagOverride,
