@@ -93,13 +93,27 @@ type AgentAPI interface {
 	UpdateTTL(id, output, status string) error
 }
 
+// ACLsAPI is the consul/api.ACL API subset used by Nomad Server.
+type ACLsAPI interface {
+	// We are looking up by [operator token] SecretID, which implies we need
+	// to use this method instead of the normal TokenRead, which can only be
+	// used to lookup tokens by their AccessorID.
+	TokenReadSelf(q *api.QueryOptions) (*api.ACLToken, *api.QueryMeta, error)
+	PolicyRead(policyID string, q *api.QueryOptions) (*api.ACLPolicy, *api.QueryMeta, error)
+	RoleRead(roleID string, q *api.QueryOptions) (*api.ACLRole, *api.QueryMeta, error)
+	TokenCreate(partial *api.ACLToken, q *api.WriteOptions) (*api.ACLToken, *api.WriteMeta, error)
+	TokenDelete(accessorID string, q *api.WriteOptions) (*api.WriteMeta, error)
+	TokenList(q *api.QueryOptions) ([]*api.ACLTokenListEntry, *api.QueryMeta, error)
+}
+
 func agentServiceUpdateRequired(reg *api.AgentServiceRegistration, svc *api.AgentService) bool {
 	return !(reg.Kind == svc.Kind &&
 		reg.ID == svc.ID &&
 		reg.Port == svc.Port &&
 		reg.Address == svc.Address &&
 		reg.Name == svc.Service &&
-		reflect.DeepEqual(reg.Tags, svc.Tags))
+		reflect.DeepEqual(reg.Tags, svc.Tags) &&
+		reflect.DeepEqual(reg.Meta, svc.Meta))
 }
 
 // operations are submitted to the main loop via commit() for synchronizing
@@ -713,9 +727,18 @@ func (c *ServiceClient) serviceRegs(ops *operations, service *structs.Service, w
 		return nil, fmt.Errorf("invalid Consul Connect configuration for service %q: %v", service.Name, err)
 	}
 
-	meta := make(map[string]string, len(service.Meta))
-	for k, v := range service.Meta {
-		meta[k] = v
+	// Determine whether to use meta or canary_meta
+	var meta map[string]string
+	if workload.Canary && len(service.CanaryMeta) > 0 {
+		meta = make(map[string]string, len(service.CanaryMeta)+1)
+		for k, v := range service.CanaryMeta {
+			meta[k] = v
+		}
+	} else {
+		meta = make(map[string]string, len(service.Meta)+1)
+		for k, v := range service.Meta {
+			meta[k] = v
+		}
 	}
 
 	// This enables the consul UI to show that Nomad registered this service
