@@ -3,7 +3,7 @@ package agent
 import (
 	"bytes"
 
-	"github.com/hashicorp/go-syslog"
+	gsyslog "github.com/hashicorp/go-syslog"
 	"github.com/hashicorp/logutils"
 )
 
