@@ -610,6 +610,26 @@ func Commands(metaPtr *Meta, agentUi cli.Ui) map[string]cli.CommandFactory {
 				Meta: meta,
 			}, nil
 		},
+		"system": func() (cli.Command, error) {
+			return &SystemCommand{
+				Meta: meta,
+			}, nil
+		},
+		"system gc": func() (cli.Command, error) {
+			return &SystemGCCommand{
+				Meta: meta,
+			}, nil
+		},
+		"system reconcile": func() (cli.Command, error) {
+			return &SystemReconcileCommand{
+				Meta: meta,
+			}, nil
+		},
+		"system reconcile summaries": func() (cli.Command, error) {
+			return &SystemReconcileSummariesCommand{
+				Meta: meta,
+			}, nil
+		},
 		"ui": func() (cli.Command, error) {
 			return &UiCommand{
 				Meta: meta,
