@@ -9,11 +9,10 @@ import (
 	"time"
 
 	humanize "github.com/dustin/go-humanize"
-	"github.com/posener/complete"
-
 	"github.com/hashicorp/nomad/api"
 	"github.com/hashicorp/nomad/api/contexts"
 	"github.com/hashicorp/nomad/helper"
+	"github.com/posener/complete"
 )
 
 const (
@@ -300,6 +299,16 @@ func nodeDrivers(n *api.Node) []string {
 	return drivers
 }
 
+func nodeVolumeNames(n *api.Node) []string {
+	var volumes []string
+	for name := range n.HostVolumes {
+		volumes = append(volumes, name)
+	}
+
+	sort.Strings(volumes)
+	return volumes
+}
+
 func formatDrain(n *api.Node) string {
 	if n.DrainStrategy != nil {
 		b := new(strings.Builder)
@@ -324,7 +333,7 @@ func formatDrain(n *api.Node) string {
 func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 	// Format the header output
 	basic := []string{
-		fmt.Sprintf("ID|%s", limit(node.ID, c.length)),
+		fmt.Sprintf("ID|%s", node.ID),
 		fmt.Sprintf("Name|%s", node.Name),
 		fmt.Sprintf("Class|%s", node.NodeClass),
 		fmt.Sprintf("DC|%s", node.Datacenter),
@@ -334,76 +343,107 @@ func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 	}
 
 	if c.short {
+		basic = append(basic, fmt.Sprintf("Host Volumes|%s", strings.Join(nodeVolumeNames(node), ",")))
 		basic = append(basic, fmt.Sprintf("Drivers|%s", strings.Join(nodeDrivers(node), ",")))
 		c.Ui.Output(c.Colorize().Color(formatKV(basic)))
-	} else {
-		// Get the host stats
-		hostStats, nodeStatsErr := client.Nodes().Stats(node.ID, nil)
-		if nodeStatsErr != nil {
-			c.Ui.Output("")
-			c.Ui.Error(fmt.Sprintf("error fetching node stats: %v", nodeStatsErr))
-		}
-		if hostStats != nil {
-			uptime := time.Duration(hostStats.Uptime * uint64(time.Second))
-			basic = append(basic, fmt.Sprintf("Uptime|%s", uptime.String()))
-		}
 
-		// Emit the driver info
-		if !c.verbose {
-			driverStatus := fmt.Sprintf("Driver Status| %s", c.outputTruncatedNodeDriverInfo(node))
-			basic = append(basic, driverStatus)
-		}
-
-		c.Ui.Output(c.Colorize().Color(formatKV(basic)))
-
-		if c.verbose {
-			c.outputNodeDriverInfo(node)
-		}
-
-		// Emit node events
-		c.outputNodeStatusEvents(node)
-
-		// Get list of running allocations on the node
-		runningAllocs, err := getRunningAllocs(client, node.ID)
-		if err != nil {
-			c.Ui.Error(fmt.Sprintf("Error querying node for running allocations: %s", err))
+		// Output alloc info
+		if err := c.outputAllocInfo(client, node); err != nil {
+			c.Ui.Error(fmt.Sprintf("%s", err))
 			return 1
 		}
 
-		allocatedResources := getAllocatedResources(client, runningAllocs, node)
-		c.Ui.Output(c.Colorize().Color("\n[bold]Allocated Resources[reset]"))
-		c.Ui.Output(formatList(allocatedResources))
+		return 0
+	}
 
-		actualResources, err := getActualResources(client, runningAllocs, node)
-		if err == nil {
-			c.Ui.Output(c.Colorize().Color("\n[bold]Allocation Resource Utilization[reset]"))
-			c.Ui.Output(formatList(actualResources))
-		}
+	// Get the host stats
+	hostStats, nodeStatsErr := client.Nodes().Stats(node.ID, nil)
+	if nodeStatsErr != nil {
+		c.Ui.Output("")
+		c.Ui.Error(fmt.Sprintf("error fetching node stats: %v", nodeStatsErr))
+	}
+	if hostStats != nil {
+		uptime := time.Duration(hostStats.Uptime * uint64(time.Second))
+		basic = append(basic, fmt.Sprintf("Uptime|%s", uptime.String()))
+	}
 
-		hostResources, err := getHostResources(hostStats, node)
-		if err != nil {
-			c.Ui.Output("")
-			c.Ui.Error(fmt.Sprintf("error fetching node stats: %v", err))
-		}
-		if err == nil {
-			c.Ui.Output(c.Colorize().Color("\n[bold]Host Resource Utilization[reset]"))
-			c.Ui.Output(formatList(hostResources))
-		}
+	// When we're not running in verbose mode, then also include host volumes and
+	// driver info in the basic output
+	if !c.verbose {
+		basic = append(basic, fmt.Sprintf("Host Volumes|%s", strings.Join(nodeVolumeNames(node), ",")))
 
-		if hostStats != nil && c.stats {
-			c.Ui.Output(c.Colorize().Color("\n[bold]CPU Stats[reset]"))
-			c.printCpuStats(hostStats)
-			c.Ui.Output(c.Colorize().Color("\n[bold]Memory Stats[reset]"))
-			c.printMemoryStats(hostStats)
-			c.Ui.Output(c.Colorize().Color("\n[bold]Disk Stats[reset]"))
-			c.printDiskStats(hostStats)
+		driverStatus := fmt.Sprintf("Driver Status| %s", c.outputTruncatedNodeDriverInfo(node))
+		basic = append(basic, driverStatus)
+	}
+
+	// Output the basic info
+	c.Ui.Output(c.Colorize().Color(formatKV(basic)))
+
+	// If we're running in verbose mode, include full host volume and driver info
+	if c.verbose {
+		c.outputNodeVolumeInfo(node)
+		c.outputNodeDriverInfo(node)
+	}
+
+	// Emit node events
+	c.outputNodeStatusEvents(node)
+
+	// Get list of running allocations on the node
+	runningAllocs, err := getRunningAllocs(client, node.ID)
+	if err != nil {
+		c.Ui.Error(fmt.Sprintf("Error querying node for running allocations: %s", err))
+		return 1
+	}
+
+	allocatedResources := getAllocatedResources(client, runningAllocs, node)
+	c.Ui.Output(c.Colorize().Color("\n[bold]Allocated Resources[reset]"))
+	c.Ui.Output(formatList(allocatedResources))
+
+	actualResources, err := getActualResources(client, runningAllocs, node)
+	if err == nil {
+		c.Ui.Output(c.Colorize().Color("\n[bold]Allocation Resource Utilization[reset]"))
+		c.Ui.Output(formatList(actualResources))
+	}
+
+	hostResources, err := getHostResources(hostStats, node)
+	if err != nil {
+		c.Ui.Output("")
+		c.Ui.Error(fmt.Sprintf("error fetching node stats: %v", err))
+	}
+	if err == nil {
+		c.Ui.Output(c.Colorize().Color("\n[bold]Host Resource Utilization[reset]"))
+		c.Ui.Output(formatList(hostResources))
+	}
+
+	if err == nil && node.NodeResources != nil && len(node.NodeResources.Devices) > 0 {
+		c.Ui.Output(c.Colorize().Color("\n[bold]Device Resource Utilization[reset]"))
+		c.Ui.Output(formatList(getDeviceResourcesForNode(hostStats.DeviceStats, node)))
+	}
+	if hostStats != nil && c.stats {
+		c.Ui.Output(c.Colorize().Color("\n[bold]CPU Stats[reset]"))
+		c.printCpuStats(hostStats)
+		c.Ui.Output(c.Colorize().Color("\n[bold]Memory Stats[reset]"))
+		c.printMemoryStats(hostStats)
+		c.Ui.Output(c.Colorize().Color("\n[bold]Disk Stats[reset]"))
+		c.printDiskStats(hostStats)
+		if len(hostStats.DeviceStats) > 0 {
+			c.Ui.Output(c.Colorize().Color("\n[bold]Device Stats[reset]"))
+			printDeviceStats(c.Ui, hostStats.DeviceStats)
 		}
 	}
 
+	if err := c.outputAllocInfo(client, node); err != nil {
+		c.Ui.Error(fmt.Sprintf("%s", err))
+		return 1
+	}
+
+	return 0
+}
+
+func (c *NodeStatusCommand) outputAllocInfo(client *api.Client, node *api.Node) error {
 	nodeAllocs, _, err := client.Nodes().Allocations(node.ID, nil)
 	if err != nil {
-		c.Ui.Error(fmt.Sprintf("Error querying node allocations: %s", err))
-		return 1
+		return fmt.Errorf("Error querying node allocations: %s", err)
 	}
 
 	c.Ui.Output(c.Colorize().Color("\n[bold]Allocations[reset]"))
@@ -411,10 +451,11 @@ func (c *NodeStatusCommand) formatNode(client *api.Client, node *api.Node) int {
 
 	if c.verbose {
 		c.formatAttributes(node)
+		c.formatDeviceAttributes(node)
 		c.formatMeta(node)
 	}
-	return 0
 
+	return nil
 }
 
 func (c *NodeStatusCommand) outputTruncatedNodeDriverInfo(node *api.Node) string {
@@ -435,6 +476,25 @@ func (c *NodeStatusCommand) outputTruncatedNodeDriverInfo(node *api.Node) string
 	return strings.Trim(strings.Join(drivers, ","), ", ")
 }
 
+func (c *NodeStatusCommand) outputNodeVolumeInfo(node *api.Node) {
+	c.Ui.Output(c.Colorize().Color("\n[bold]Host Volumes"))
+
+	names := make([]string, 0, len(node.HostVolumes))
+	for name := range node.HostVolumes {
+		names = append(names, name)
+	}
+	sort.Strings(names)
+
+	output := make([]string, 0, len(names)+1)
+	output = append(output, "Name|ReadOnly|Source")
+
+	for _, volName := range names {
+		info := node.HostVolumes[volName]
+		output = append(output, fmt.Sprintf("%s|%v|%s", volName, info.ReadOnly, info.Path))
+	}
+	c.Ui.Output(formatList(output))
+}
+
 func (c *NodeStatusCommand) outputNodeDriverInfo(node *api.Node) {
 	c.Ui.Output(c.Colorize().Color("\n[bold]Drivers"))
 
@@ -521,6 +581,35 @@ func (c *NodeStatusCommand) formatAttributes(node *api.Node) {
 	c.Ui.Output(formatKV(attributes))
 }
 
+func (c *NodeStatusCommand) formatDeviceAttributes(node *api.Node) {
+	if node.NodeResources == nil {
+		return
+	}
+	devices := node.NodeResources.Devices
+	if len(devices) == 0 {
+		return
+	}
+
+	sort.Slice(devices, func(i, j int) bool {
+		return devices[i].ID() < devices[j].ID()
+	})
+
+	first := true
+	for _, d := range devices {
+		if len(d.Attributes) == 0 {
+			continue
+		}
+
+		if first {
+			c.Ui.Output("\nDevice Group Attributes")
+			first = false
+		} else {
+			c.Ui.Output("")
+		}
+		c.Ui.Output(formatKV(getDeviceAttributes(d)))
+	}
+}
+
 func (c *NodeStatusCommand) formatMeta(node *api.Node) {
 	// Print the meta
 	keys := make([]string, 0, len(node.Meta))
@@ -603,25 +692,22 @@ func getAllocatedResources(client *api.Client, runningAllocs []*api.Allocation,
 	total := computeNodeTotalResources(node)
 
 	// Get Resources
-	var cpu, mem, disk, iops int
+	var cpu, mem, disk int
 	for _, alloc := range runningAllocs {
 		cpu += *alloc.Resources.CPU
 		mem += *alloc.Resources.MemoryMB
 		disk += *alloc.Resources.DiskMB
-		iops += *alloc.Resources.IOPS
 	}
 
 	resources := make([]string, 2)
-	resources[0] = "CPU|Memory|Disk|IOPS"
-	resources[1] = fmt.Sprintf("%d/%d MHz|%s/%s|%s/%s|%d/%d",
+	resources[0] = "CPU|Memory|Disk"
+	resources[1] = fmt.Sprintf("%d/%d MHz|%s/%s|%s/%s",
 		cpu,
 		*total.CPU,
 		humanize.IBytes(uint64(mem*bytesPerMegabyte)),
 		humanize.IBytes(uint64(*total.MemoryMB*bytesPerMegabyte)),
 		humanize.IBytes(uint64(disk*bytesPerMegabyte)),
-		humanize.IBytes(uint64(*total.DiskMB*bytesPerMegabyte)),
-		iops,
-		*total.IOPS)
+		humanize.IBytes(uint64(*total.DiskMB*bytesPerMegabyte)))
 
 	return resources
 }
@@ -639,7 +725,6 @@ func computeNodeTotalResources(node *api.Node) api.Resources {
 	total.CPU = helper.IntToPtr(*r.CPU - *res.CPU)
 	total.MemoryMB = helper.IntToPtr(*r.MemoryMB - *res.MemoryMB)
 	total.DiskMB = helper.IntToPtr(*r.DiskMB - *res.DiskMB)
-	total.IOPS = helper.IntToPtr(*r.IOPS - *res.IOPS)
 	return total
 }
 
