@@ -17,6 +17,7 @@ import (
 	docker "github.com/fsouza/go-dockerclient"
 	"github.com/hashicorp/nomad/client/allocdir"
 	"github.com/hashicorp/nomad/client/testutil"
+	"github.com/hashicorp/nomad/helper/freeport"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/plugins/drivers"
 	dtestutil "github.com/hashicorp/nomad/plugins/drivers/testutils"
@@ -30,7 +31,8 @@ func TestDockerDriver_User(t *testing.T) {
 		t.Parallel()
 	}
 	testutil.DockerCompatible(t)
-	task, cfg, _ := dockerTask(t)
+	task, cfg, ports := dockerTask(t)
+	defer freeport.Return(ports)
 	task.User = "alice"
 	cfg.Command = "/bin/sleep"
 	cfg.Args = []string{"10000"}
@@ -151,7 +153,8 @@ func TestDockerDriver_CPUCFSPeriod(t *testing.T) {
 	}
 	testutil.DockerCompatible(t)
 
-	task, cfg, _ := dockerTask(t)
+	task, cfg, ports := dockerTask(t)
+	defer freeport.Return(ports)
 	cfg.CPUHardLimit = true
 	cfg.CPUCFSPeriod = 1000000
 	require.NoError(t, task.EncodeConcreteDriverConfig(cfg))
@@ -169,7 +172,8 @@ func TestDockerDriver_CPUCFSPeriod(t *testing.T) {
 
 func TestDockerDriver_Sysctl_Ulimit(t *testing.T) {
 	testutil.DockerCompatible(t)
-	task, cfg, _ := dockerTask(t)
+	task, cfg, ports := dockerTask(t)
+	defer freeport.Return(ports)
 	expectedUlimits := map[string]string{
 		"nproc":  "4242",
 		"nofile": "2048:4096",
@@ -237,7 +241,7 @@ func TestDockerDriver_Sysctl_Ulimit_Errors(t *testing.T) {
 	}
 
 	for _, tc := range testCases {
-		task, cfg, _ := dockerTask(t)
+		task, cfg, ports := dockerTask(t)
 		cfg.Ulimit = tc.ulimitConfig
 		require.NoError(t, task.EncodeConcreteDriverConfig(cfg))
 
@@ -249,6 +253,7 @@ func TestDockerDriver_Sysctl_Ulimit_Errors(t *testing.T) {
 		_, _, err := d.StartTask(task)
 		require.NotNil(t, err, "Expected non nil error")
 		require.Contains(t, err.Error(), tc.err.Error())
+		freeport.Return(ports)
 	}
 }
 
@@ -336,7 +341,8 @@ func TestDockerDriver_BindMountsHonorVolumesEnabledFlag(t *testing.T) {
 
 		for _, c := range cases {
 			t.Run(c.name, func(t *testing.T) {
-				task, cfg, _ := dockerTask(t)
+				task, cfg, ports := dockerTask(t)
+				defer freeport.Return(ports)
 				cfg.VolumeDriver = c.volumeDriver
 				cfg.Volumes = c.volumes
 
@@ -362,7 +368,8 @@ func TestDockerDriver_BindMountsHonorVolumesEnabledFlag(t *testing.T) {
 
 		for _, c := range cases {
 			t.Run(c.name, func(t *testing.T) {
-				task, cfg, _ := dockerTask(t)
+				task, cfg, ports := dockerTask(t)
+				defer freeport.Return(ports)
 				cfg.VolumeDriver = c.volumeDriver
 				cfg.Volumes = c.volumes
 
@@ -509,7 +516,8 @@ func TestDockerDriver_MountsSerialization(t *testing.T) {
 
 		for _, c := range cases {
 			t.Run(c.name, func(t *testing.T) {
-				task, cfg, _ := dockerTask(t)
+				task, cfg, ports := dockerTask(t)
+				defer freeport.Return(ports)
 				cfg.Mounts = c.passedMounts
 
 				task.AllocDir = allocDir
@@ -531,7 +539,8 @@ func TestDockerDriver_MountsSerialization(t *testing.T) {
 
 		for _, c := range cases {
 			t.Run(c.name, func(t *testing.T) {
-				task, cfg, _ := dockerTask(t)
+				task, cfg, ports := dockerTask(t)
+				defer freeport.Return(ports)
 				cfg.Mounts = c.passedMounts
 
 				task.AllocDir = allocDir
@@ -559,7 +568,8 @@ func TestDockerDriver_CreateContainerConfig_MountsCombined(t *testing.T) {
 	t.Parallel()
 	testutil.DockerCompatible(t)
 
-	task, cfg, _ := dockerTask(t)
+	task, cfg, ports := dockerTask(t)
+	defer freeport.Return(ports)
 
 	task.Devices = []*drivers.DeviceConfig{
 		{
