@@ -77,14 +77,15 @@ func testCleanupDeadServer(t *testing.T, raftVersion int) {
 		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = raft.ProtocolVersion(raftVersion)
 	}
-	s1 := TestServer(t, conf)
-	defer s1.Shutdown()
 
-	s2 := TestServer(t, conf)
-	defer s2.Shutdown()
+	s1, cleanupS1 := TestServer(t, conf)
+	defer cleanupS1()
 
-	s3 := TestServer(t, conf)
-	defer s3.Shutdown()
+	s2, cleanupS2 := TestServer(t, conf)
+	defer cleanupS2()
+
+	s3, cleanupS3 := TestServer(t, conf)
+	defer cleanupS3()
 
 	servers := []*Server{s1, s2, s3}
 
@@ -96,8 +97,8 @@ func testCleanupDeadServer(t *testing.T, raftVersion int) {
 	}
 
 	// Bring up a new server
-	s4 := TestServer(t, conf)
-	defer s4.Shutdown()
+	s4, cleanupS4 := TestServer(t, conf)
+	defer cleanupS4()
 
 	// Kill a non-leader server
 	s3.Shutdown()
@@ -125,24 +126,25 @@ func testCleanupDeadServer(t *testing.T, raftVersion int) {
 
 func TestAutopilot_CleanupDeadServerPeriodic(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
 
 	conf := func(c *Config) {
 		c.DevDisableBootstrap = true
 	}
 
-	s2 := TestServer(t, conf)
-	defer s2.Shutdown()
+	s2, cleanupS2 := TestServer(t, conf)
+	defer cleanupS2()
 
-	s3 := TestServer(t, conf)
-	defer s3.Shutdown()
+	s3, cleanupS3 := TestServer(t, conf)
+	defer cleanupS3()
 
-	s4 := TestServer(t, conf)
-	defer s4.Shutdown()
+	s4, cleanupS4 := TestServer(t, conf)
+	defer cleanupS4()
 
-	s5 := TestServer(t, conf)
-	defer s5.Shutdown()
+	s5, cleanupS5 := TestServer(t, conf)
+	defer cleanupS5()
 
 	servers := []*Server{s1, s2, s3, s4, s5}
 
@@ -171,21 +173,22 @@ func TestAutopilot_CleanupDeadServerPeriodic(t *testing.T) {
 
 func TestAutopilot_RollingUpdate(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.RaftConfig.ProtocolVersion = 3
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
 	conf := func(c *Config) {
 		c.DevDisableBootstrap = true
 		c.RaftConfig.ProtocolVersion = 3
 	}
 
-	s2 := TestServer(t, conf)
-	defer s2.Shutdown()
+	s2, cleanupS2 := TestServer(t, conf)
+	defer cleanupS2()
 
-	s3 := TestServer(t, conf)
-	defer s3.Shutdown()
+	s3, cleanupS3 := TestServer(t, conf)
+	defer cleanupS3()
 
 	// Join the servers to s1, and wait until they are all promoted to
 	// voters.
@@ -199,8 +202,8 @@ func TestAutopilot_RollingUpdate(t *testing.T) {
 	})
 
 	// Add one more server like we are doing a rolling update.
-	s4 := TestServer(t, conf)
-	defer s4.Shutdown()
+	s4, cleanupS4 := TestServer(t, conf)
+	defer cleanupS4()
 	TestJoin(t, s1, s4)
 	servers = append(servers, s4)
 	retry.Run(t, func(r *retry.R) {
@@ -243,22 +246,22 @@ func TestAutopilot_RollingUpdate(t *testing.T) {
 
 func TestAutopilot_CleanupStaleRaftServer(t *testing.T) {
 	t.Skip("TestAutopilot_CleanupDeadServer is very flaky, removing it for now")
-
 	t.Parallel()
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
 
 	conf := func(c *Config) {
 		c.DevDisableBootstrap = true
 	}
-	s2 := TestServer(t, conf)
-	defer s2.Shutdown()
+	s2, cleanupS2 := TestServer(t, conf)
+	defer cleanupS2()
 
-	s3 := TestServer(t, conf)
-	defer s3.Shutdown()
+	s3, cleanupS3 := TestServer(t, conf)
+	defer cleanupS3()
 
-	s4 := TestServer(t, conf)
-	defer s4.Shutdown()
+	s4, cleanupS4 := TestServer(t, conf)
+	defer cleanupS4()
 
 	servers := []*Server{s1, s2, s3}
 
@@ -291,19 +294,20 @@ func TestAutopilot_CleanupStaleRaftServer(t *testing.T) {
 
 func TestAutopilot_PromoteNonVoter(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.RaftConfig.ProtocolVersion = 3
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 	testutil.WaitForLeader(t, s1.RPC)
 
-	s2 := TestServer(t, func(c *Config) {
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.DevDisableBootstrap = true
 		c.RaftConfig.ProtocolVersion = 3
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	TestJoin(t, s1, s2)
 
 	// Make sure we see it as a nonvoter initially. We wait until half
