@@ -4,8 +4,9 @@ import (
 	"sync"
 	"time"
 
-	"github.com/armon/go-metrics"
+	metrics "github.com/armon/go-metrics"
 	"github.com/hashicorp/consul/lib"
+	log "github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/nomad/structs"
 )
@@ -30,6 +31,9 @@ const (
 // allocations. It is unblocked when the capacity of a node that could run the
 // failed allocation becomes available.
 type BlockedEvals struct {
+	// logger is the logger to use by the blocked eval tracker.
+	logger log.Logger
+
 	evalBroker *EvalBroker
 	enabled    bool
 	stats      *BlockedStats
@@ -43,6 +47,10 @@ type BlockedEvals struct {
 	// classes.
 	escaped map[string]wrappedEval
 
+	// system is the set of system evaluations that failed to start on nodes because of
+	// resource constraints.
+	system *systemEvals
+
 	// unblockCh is used to buffer unblocking of evaluations.
 	capacityChangeCh chan *capacityUpdate
 
@@ -103,11 +111,13 @@ type BlockedStats struct {
 
 // NewBlockedEvals creates a new blocked eval tracker that will enqueue
 // unblocked evals into the passed broker.
-func NewBlockedEvals(evalBroker *EvalBroker) *BlockedEvals {
+func NewBlockedEvals(evalBroker *EvalBroker, logger log.Logger) *BlockedEvals {
 	return &BlockedEvals{
+		logger:           logger.Named("blocked_evals"),
 		evalBroker:       evalBroker,
 		captured:         make(map[string]wrappedEval),
 		escaped:          make(map[string]wrappedEval),
+		system:           newSystemEvals(),
 		jobs:             make(map[structs.NamespacedID]string),
 		unblockIndexes:   make(map[string]uint64),
 		capacityChangeCh: make(chan *capacityUpdate, unblockBuffer),
@@ -133,8 +143,8 @@ func (b *BlockedEvals) SetEnabled(enabled bool) {
 		b.l.Unlock()
 		return
 	} else if enabled {
-		go b.watchCapacity()
-		go b.prune()
+		go b.watchCapacity(b.stopCh, b.capacityChangeCh)
+		go b.prune(b.stopCh)
 	} else {
 		close(b.stopCh)
 	}
@@ -222,6 +232,12 @@ func (b *BlockedEvals) processBlock(eval *structs.Evaluation, token string) {
 		return
 	}
 
+	// System evals are indexed by node and re-processed on utilization changes in
+	// existing nodes
+	if eval.Type == structs.JobTypeSystem {
+		b.system.Add(eval, token)
+	}
+
 	// Add the eval to the set of blocked evals whose jobs constraints are
 	// captured by computed node class.
 	b.captured[eval.ID] = wrapped
@@ -257,6 +273,7 @@ func (b *BlockedEvals) processBlockJobDuplicate(eval *structs.Evaluation) (newCa
 		existingW, ok = b.escaped[existingID]
 		if !ok {
 			// This is a programming error
+			b.logger.Error("existing blocked evaluation is neither tracked as captured or escaped", "existing_id", existingID)
 			delete(b.jobs, structs.NewNamespacedID(eval.JobID, eval.Namespace))
 			return
 		}
@@ -359,6 +376,14 @@ func (b *BlockedEvals) Untrack(jobID, namespace string) {
 
 	nsID := structs.NewNamespacedID(jobID, namespace)
 
+	if evals, ok := b.system.JobEvals(nsID); ok {
+		for _, e := range evals {
+			b.system.Remove(e)
+			b.stats.TotalBlocked--
+		}
+		return
+	}
+
 	// Get the evaluation ID to cancel
 	evalID, ok := b.jobs[nsID]
 	if !ok {
@@ -458,23 +483,48 @@ func (b *BlockedEvals) UnblockClassAndQuota(class, quota string, index uint64) {
 		b.unblockIndexes[quota] = index
 	}
 	b.unblockIndexes[class] = index
+
+	// Capture chan inside the lock to prevent a race with it getting reset
+	// in Flush.
+	ch := b.capacityChangeCh
 	b.l.Unlock()
 
-	b.capacityChangeCh <- &capacityUpdate{
+	ch <- &capacityUpdate{
 		computedClass: class,
 		quotaChange:   quota,
 		index:         index,
 	}
 }
 
+// UnblockNode finds any blocked evalution that's node specific (system jobs) and enqueues
+// it on the eval broker
+func (b *BlockedEvals) UnblockNode(nodeID string, index uint64) {
+	b.l.Lock()
+	defer b.l.Unlock()
+
+	evals, ok := b.system.NodeEvals(nodeID)
+
+	// Do nothing if not enabled
+	if !b.enabled || !ok || len(evals) == 0 {
+		return
+	}
+
+	for e := range evals {
+		b.system.Remove(e)
+		b.stats.TotalBlocked--
+	}
+
+	b.evalBroker.EnqueueAll(evals)
+}
+
 // watchCapacity is a long lived function that watches for capacity changes in
 // nodes and unblocks the correct set of evals.
-func (b *BlockedEvals) watchCapacity() {
+func (b *BlockedEvals) watchCapacity(stopCh <-chan struct{}, changeCh <-chan *capacityUpdate) {
 	for {
 		select {
-		case <-b.stopCh:
+		case <-stopCh:
 			return
-		case update := <-b.capacityChangeCh:
+		case update := <-changeCh:
 			b.unblock(update.computedClass, update.quotaChange, update.index)
 		}
 	}
@@ -600,6 +650,11 @@ SCAN:
 		b.l.Unlock()
 		return dups
 	}
+
+	// Capture chans inside the lock to prevent a race with them getting
+	// reset in Flush
+	dupCh := b.duplicateCh
+	stopCh := b.stopCh
 	b.l.Unlock()
 
 	// Create the timer
@@ -610,11 +665,11 @@ SCAN:
 	}
 
 	select {
-	case <-b.stopCh:
+	case <-stopCh:
 		return nil
 	case <-timeoutCh:
 		return nil
-	case <-b.duplicateCh:
+	case <-dupCh:
 		goto SCAN
 	}
 }
@@ -637,6 +692,7 @@ func (b *BlockedEvals) Flush() {
 	b.capacityChangeCh = make(chan *capacityUpdate, unblockBuffer)
 	b.stopCh = make(chan struct{})
 	b.duplicateCh = make(chan struct{}, 1)
+	b.system = newSystemEvals()
 }
 
 // Stats is used to query the state of the blocked eval tracker.
@@ -655,7 +711,7 @@ func (b *BlockedEvals) Stats() *BlockedStats {
 }
 
 // EmitStats is used to export metrics about the blocked eval tracker while enabled
-func (b *BlockedEvals) EmitStats(period time.Duration, stopCh chan struct{}) {
+func (b *BlockedEvals) EmitStats(period time.Duration, stopCh <-chan struct{}) {
 	for {
 		select {
 		case <-time.After(period):
@@ -670,13 +726,13 @@ func (b *BlockedEvals) EmitStats(period time.Duration, stopCh chan struct{}) {
 }
 
 // prune is a long lived function that prunes unnecessary objects on a timer.
-func (b *BlockedEvals) prune() {
+func (b *BlockedEvals) prune(stopCh <-chan struct{}) {
 	ticker := time.NewTicker(pruneInterval)
 	defer ticker.Stop()
 
 	for {
 		select {
-		case <-b.stopCh:
+		case <-stopCh:
 			return
 		case <-ticker.C:
 			b.pruneUnblockIndexes()
