@@ -3,17 +3,16 @@ package deploymentwatcher
 import (
 	"context"
 	"fmt"
-	"log"
 	"sync"
 	"time"
 
-	"golang.org/x/time/rate"
-
+	log "github.com/hashicorp/go-hclog"
 	memdb "github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/state"
 	"github.com/hashicorp/nomad/nomad/structs"
+	"golang.org/x/time/rate"
 )
 
 const (
@@ -91,7 +90,7 @@ type deploymentWatcher struct {
 	// by holding the lock or using the setter and getter methods.
 	latestEval uint64
 
-	logger *log.Logger
+	logger log.Logger
 	ctx    context.Context
 	exitFn context.CancelFunc
 	l      sync.RWMutex
@@ -100,7 +99,7 @@ type deploymentWatcher struct {
 // newDeploymentWatcher returns a deployment watcher that is used to watch
 // deployments and trigger the scheduler as needed.
 func newDeploymentWatcher(parent context.Context, queryLimiter *rate.Limiter,
-	logger *log.Logger, state *state.StateStore, d *structs.Deployment,
+	logger log.Logger, state *state.StateStore, d *structs.Deployment,
 	j *structs.Job, triggers deploymentTriggers) *deploymentWatcher {
 
 	ctx, exitFn := context.WithCancel(parent)
@@ -112,7 +111,7 @@ func newDeploymentWatcher(parent context.Context, queryLimiter *rate.Limiter,
 		j:                  j,
 		state:              state,
 		deploymentTriggers: triggers,
-		logger:             logger,
+		logger:             logger.With("deployment_id", d.ID, "job", j.NamespacedID()),
 		ctx:                ctx,
 		exitFn:             exitFn,
 	}
@@ -250,7 +249,7 @@ func (w *deploymentWatcher) PromoteDeployment(
 	// Create the request
 	areq := &structs.ApplyDeploymentPromoteRequest{
 		DeploymentPromoteRequest: *req,
-		Eval: w.getEval(),
+		Eval:                     w.getEval(),
 	}
 
 	index, err := w.upsertDeploymentPromotion(areq)
@@ -266,6 +265,38 @@ func (w *deploymentWatcher) PromoteDeployment(
 	return nil
 }
 
+// autoPromoteDeployment creates a synthetic promotion request, and upserts it for processing
+func (w *deploymentWatcher) autoPromoteDeployment(allocs []*structs.AllocListStub) error {
+	d := w.getDeployment()
+	if !d.HasPlacedCanaries() || !d.RequiresPromotion() {
+		return nil
+	}
+
+	// AutoPromote iff every task group is marked auto_promote and is healthy. The whole
+	// job version has been incremented, so we promote together. See also AutoRevert
+	for _, tv := range d.TaskGroups {
+		if !tv.AutoPromote || tv.DesiredCanaries != len(tv.PlacedCanaries) {
+			return nil
+		}
+
+		// Find the health status of each canary
+		for _, c := range tv.PlacedCanaries {
+			for _, a := range allocs {
+				if c == a.ID && !a.DeploymentStatus.IsHealthy() {
+					return nil
+				}
+			}
+		}
+	}
+
+	// Send the request
+	_, err := w.upsertDeploymentPromotion(&structs.ApplyDeploymentPromoteRequest{
+		DeploymentPromoteRequest: structs.DeploymentPromoteRequest{DeploymentID: d.GetID(), All: true},
+		Eval:                     w.getEval(),
+	})
+	return err
+}
+
 func (w *deploymentWatcher) PauseDeployment(
 	req *structs.DeploymentPauseRequest,
 	resp *structs.DeploymentUpdateResponse) error {
@@ -382,6 +413,7 @@ FAIL:
 	for {
 		select {
 		case <-w.ctx.Done():
+			// This is the successful case, and we stop the loop
 			return
 		case <-deadlineTimer.C:
 			// We have hit the progress deadline so fail the deployment. We need
@@ -391,14 +423,14 @@ FAIL:
 			deadlineHit = true
 			fail, rback, err := w.shouldFail()
 			if err != nil {
-				w.logger.Printf("[ERR] nomad.deployment_watcher: failed to determine whether to rollback job for deployment %q: %v", w.deploymentID, err)
+				w.logger.Error("failed to determine whether to rollback job", "error", err)
 			}
 			if !fail {
-				w.logger.Printf("[DEBUG] nomad.deployment_watcher: skipping deadline for deployment %q", w.deploymentID)
+				w.logger.Debug("skipping deadline")
 				continue
 			}
 
-			w.logger.Printf("[DEBUG] nomad.deployment_watcher: deadline for deployment %q hit and rollback is %v", w.deploymentID, rback)
+			w.logger.Debug("deadline hit", "rollback", rback)
 			rollback = rback
 			break FAIL
 		case <-w.deploymentUpdateCh:
@@ -437,7 +469,7 @@ FAIL:
 					return
 				}
 
-				w.logger.Printf("[ERR] nomad.deployment_watcher: failed to retrieve allocations for deployment %q: %v", w.deploymentID, err)
+				w.logger.Error("failed to retrieve allocations", "error", err)
 				return
 			}
 			allocIndex = updates.index
@@ -450,7 +482,7 @@ FAIL:
 					return
 				}
 
-				w.logger.Printf("[ERR] nomad.deployment_watcher: failed handling allocation updates: %v", err)
+				w.logger.Error("failed handling allocation updates", "error", err)
 				return
 			}
 
@@ -461,6 +493,12 @@ FAIL:
 				break FAIL
 			}
 
+			// If permitted, automatically promote this canary deployment
+			err = w.autoPromoteDeployment(updates.allocs)
+			if err != nil {
+				w.logger.Error("failed to auto promote deployment", "error", err)
+			}
+
 			// Create an eval to push the deployment along
 			if res.createEval || len(res.allowReplacements) != 0 {
 				w.createBatchedUpdate(res.allowReplacements, allocIndex)
@@ -480,7 +518,7 @@ FAIL:
 		var err error
 		j, err = w.latestStableJob()
 		if err != nil {
-			w.logger.Printf("[ERR] nomad.deployment_watcher: failed to lookup latest stable job for %q: %v", w.j.ID, err)
+			w.logger.Error("failed to lookup latest stable job", "error", err)
 		}
 
 		// Description should include that the job is being rolled back to
@@ -496,7 +534,7 @@ FAIL:
 	e := w.getEval()
 	u := w.getDeploymentStatusUpdate(structs.DeploymentStatusFailed, desc)
 	if _, err := w.upsertDeploymentStatusUpdate(u, e, j); err != nil {
-		w.logger.Printf("[ERR] nomad.deployment_watcher: failed to update deployment %q status: %v", w.deploymentID, err)
+		w.logger.Error("failed to update deployment status", "error", err)
 	}
 }
 
@@ -746,13 +784,14 @@ func (w *deploymentWatcher) createBatchedUpdate(allowReplacements []string, forI
 
 		// Create the eval
 		if _, err := w.createUpdate(replacements, w.getEval()); err != nil {
-			w.logger.Printf("[ERR] nomad.deployment_watcher: failed to create evaluation for deployment %q: %v", w.deploymentID, err)
+			w.logger.Error("failed to create evaluation for deployment", "deployment_id", w.deploymentID, "error", err)
 		}
 	})
 }
 
 // getEval returns an evaluation suitable for the deployment
 func (w *deploymentWatcher) getEval() *structs.Evaluation {
+	now := time.Now().UTC().UnixNano()
 	return &structs.Evaluation{
 		ID:           uuid.Generate(),
 		Namespace:    w.j.Namespace,
@@ -762,6 +801,8 @@ func (w *deploymentWatcher) getEval() *structs.Evaluation {
 		JobID:        w.j.ID,
 		DeploymentID: w.deploymentID,
 		Status:       structs.EvalStatusPending,
+		CreateTime:   now,
+		ModifyTime:   now,
 	}
 }
 
@@ -780,8 +821,10 @@ type allocUpdates struct {
 	err    error
 }
 
-// getAllocsCh retrieves the allocations that are part of the deployment blocking
-// at the given index.
+// getAllocsCh creates a channel and starts a goroutine that
+// 1. parks a blocking query for allocations on the state
+// 2. reads those and drops them on the channel
+// This query runs once here, but watch calls it in a loop
 func (w *deploymentWatcher) getAllocsCh(index uint64) <-chan *allocUpdates {
 	out := make(chan *allocUpdates, 1)
 	go func() {
