@@ -15,6 +15,7 @@ import (
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/scheduler"
 	"github.com/hashicorp/raft"
+	"github.com/pkg/errors"
 	"github.com/ugorji/go/codec"
 )
 
@@ -45,6 +46,8 @@ const (
 	ACLPolicySnapshot
 	ACLTokenSnapshot
 	SchedulerConfigSnapshot
+	ClusterMetadataSnapshot
+	ServiceIdentityTokenAccessorSnapshot
 )
 
 // LogApplier is the definition of a function that can apply a Raft log
@@ -251,6 +254,12 @@ func (n *nomadFSM) Apply(log *raft.Log) interface{} {
 		return n.applySchedulerConfigUpdate(buf[1:], log.Index)
 	case structs.NodeBatchDeregisterRequestType:
 		return n.applyDeregisterNodeBatch(buf[1:], log.Index)
+	case structs.ClusterMetadataRequestType:
+		return n.applyClusterMetadata(buf[1:], log.Index)
+	case structs.ServiceIdentityAccessorRegisterRequestType:
+		return n.applyUpsertSIAccessor(buf[1:], log.Index)
+	case structs.ServiceIdentityAccessorDeregisterRequestType:
+		return n.applyDeregisterSIAccessor(buf[1:], log.Index)
 	}
 
 	// Check enterprise only message types.
@@ -267,6 +276,24 @@ func (n *nomadFSM) Apply(log *raft.Log) interface{} {
 	panic(fmt.Errorf("failed to apply request: %#v", buf))
 }
 
+func (n *nomadFSM) applyClusterMetadata(buf []byte, index uint64) interface{} {
+	defer metrics.MeasureSince([]string{"nomad", "fsm", "cluster_meta"}, time.Now())
+
+	var req structs.ClusterMetadata
+	if err := structs.Decode(buf, &req); err != nil {
+		panic(fmt.Errorf("failed to decode request: %v", err))
+	}
+
+	if err := n.state.ClusterSetMetadata(index, &req); err != nil {
+		n.logger.Error("ClusterSetMetadata failed", "error", err)
+		return err
+	}
+
+	n.logger.Trace("ClusterSetMetadata", "cluster_id", req.ClusterID, "create_time", req.CreateTime)
+
+	return nil
+}
+
 func (n *nomadFSM) applyUpsertNode(buf []byte, index uint64) interface{} {
 	defer metrics.MeasureSince([]string{"nomad", "fsm", "register_node"}, time.Now())
 	var req structs.NodeRegisterRequest
@@ -681,22 +708,23 @@ func (n *nomadFSM) applyAllocUpdate(buf []byte, index uint64) interface{} {
 	// prior to being inserted into MemDB.
 	structs.DenormalizeAllocationJobs(req.Job, req.Alloc)
 
-	// COMPAT(0.11): Remove in 0.11
-	// Calculate the total resources of allocations. It is pulled out in the
-	// payload to avoid encoding something that can be computed, but should be
-	// denormalized prior to being inserted into MemDB.
 	for _, alloc := range req.Alloc {
-		if alloc.Resources != nil {
-			continue
+		// COMPAT(0.11): Remove in 0.11
+		// Calculate the total resources of allocations. It is pulled out in the
+		// payload to avoid encoding something that can be computed, but should be
+		// denormalized prior to being inserted into MemDB.
+		if alloc.Resources == nil {
+			alloc.Resources = new(structs.Resources)
+			for _, task := range alloc.TaskResources {
+				alloc.Resources.Add(task)
+			}
+
+			// Add the shared resources
+			alloc.Resources.Add(alloc.SharedResources)
 		}
 
-		alloc.Resources = new(structs.Resources)
-		for _, task := range alloc.TaskResources {
-			alloc.Resources.Add(task)
-		}
-
-		// Add the shared resources
-		alloc.Resources.Add(alloc.SharedResources)
+		// Handle upgrade path
+		alloc.Canonicalize()
 	}
 
 	if err := n.state.UpsertAllocs(index, req.Alloc); err != nil {
@@ -844,6 +872,36 @@ func (n *nomadFSM) applyDeregisterVaultAccessor(buf []byte, index uint64) interf
 	return nil
 }
 
+func (n *nomadFSM) applyUpsertSIAccessor(buf []byte, index uint64) interface{} {
+	defer metrics.MeasureSince([]string{"nomad", "fsm", "upsert_si_accessor"}, time.Now())
+	var request structs.SITokenAccessorsRequest
+	if err := structs.Decode(buf, &request); err != nil {
+		panic(errors.Wrap(err, "failed to decode request"))
+	}
+
+	if err := n.state.UpsertSITokenAccessors(index, request.Accessors); err != nil {
+		n.logger.Error("UpsertSITokenAccessors failed", "error", err)
+		return err
+	}
+
+	return nil
+}
+
+func (n *nomadFSM) applyDeregisterSIAccessor(buf []byte, index uint64) interface{} {
+	defer metrics.MeasureSince([]string{"nomad", "fsm", "deregister_si_accessor"}, time.Now())
+	var request structs.SITokenAccessorsRequest
+	if err := structs.Decode(buf, &request); err != nil {
+		panic(errors.Wrap(err, "failed to decode request"))
+	}
+
+	if err := n.state.DeleteSITokenAccessors(index, request.Accessors); err != nil {
+		n.logger.Error("DeregisterSITokenAccessor failed", "error", err)
+		return err
+	}
+
+	return nil
+}
+
 // applyPlanApply applies the results of a plan application
 func (n *nomadFSM) applyPlanResults(buf []byte, index uint64) interface{} {
 	defer metrics.MeasureSince([]string{"nomad", "fsm", "apply_plan_results"}, time.Now())
@@ -1166,6 +1224,9 @@ func (n *nomadFSM) Restore(old io.ReadCloser) error {
 				return err
 			}
 
+			// Handle upgrade path
+			alloc.Canonicalize()
+
 			if err := restore.AllocRestore(alloc); err != nil {
 				return err
 			}
@@ -1208,6 +1269,15 @@ func (n *nomadFSM) Restore(old io.ReadCloser) error {
 				return err
 			}
 
+		case ServiceIdentityTokenAccessorSnapshot:
+			accessor := new(structs.SITokenAccessor)
+			if err := dec.Decode(accessor); err != nil {
+				return err
+			}
+			if err := restore.SITokenAccessorRestore(accessor); err != nil {
+				return err
+			}
+
 		case JobVersionSnapshot:
 			version := new(structs.Job)
 			if err := dec.Decode(version); err != nil {
@@ -1255,6 +1325,15 @@ func (n *nomadFSM) Restore(old io.ReadCloser) error {
 				return err
 			}
 
+		case ClusterMetadataSnapshot:
+			meta := new(structs.ClusterMetadata)
+			if err := dec.Decode(meta); err != nil {
+				return err
+			}
+			if err := restore.ClusterMetadataRestore(meta); err != nil {
+				return err
+			}
+
 		default:
 			// Check if this is an enterprise only object being restored
 			restorer, ok := n.enterpriseRestorers[snapType]
@@ -1503,6 +1582,10 @@ func (s *nomadSnapshot) Persist(sink raft.SnapshotSink) error {
 		sink.Cancel()
 		return err
 	}
+	if err := s.persistSITokenAccessors(sink, encoder); err != nil {
+		sink.Cancel()
+		return err
+	}
 	if err := s.persistJobVersions(sink, encoder); err != nil {
 		sink.Cancel()
 		return err
@@ -1527,6 +1610,10 @@ func (s *nomadSnapshot) Persist(sink raft.SnapshotSink) error {
 		sink.Cancel()
 		return err
 	}
+	if err := s.persistClusterMetadata(sink, encoder); err != nil {
+		sink.Cancel()
+		return err
+	}
 	return nil
 }
 
@@ -1747,6 +1834,23 @@ func (s *nomadSnapshot) persistVaultAccessors(sink raft.SnapshotSink,
 	return nil
 }
 
+func (s *nomadSnapshot) persistSITokenAccessors(sink raft.SnapshotSink, encoder *codec.Encoder) error {
+	ws := memdb.NewWatchSet()
+	accessors, err := s.snap.SITokenAccessors(ws)
+	if err != nil {
+		return err
+	}
+
+	for raw := accessors.Next(); raw != nil; raw = accessors.Next() {
+		accessor := raw.(*structs.SITokenAccessor)
+		sink.Write([]byte{byte(ServiceIdentityTokenAccessorSnapshot)})
+		if err := encoder.Encode(accessor); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (s *nomadSnapshot) persistJobVersions(sink raft.SnapshotSink,
 	encoder *codec.Encoder) error {
 	// Get all the jobs
@@ -1874,6 +1978,24 @@ func (s *nomadSnapshot) persistSchedulerConfig(sink raft.SnapshotSink,
 	return nil
 }
 
+func (s *nomadSnapshot) persistClusterMetadata(sink raft.SnapshotSink,
+	encoder *codec.Encoder) error {
+
+	// Get the cluster metadata
+	clusterMetadata, err := s.snap.ClusterMetadata()
+	if err != nil {
+		return err
+	}
+
+	// Write out the cluster metadata
+	sink.Write([]byte{byte(ClusterMetadataSnapshot)})
+	if err := encoder.Encode(clusterMetadata); err != nil {
+		return err
+	}
+
+	return nil
+}
+
 // Release is a no-op, as we just need to GC the pointer
 // to the state store snapshot. There is nothing to explicitly
 // cleanup.
