@@ -20,6 +20,7 @@ import (
 	"github.com/hashicorp/nomad/nomad/state"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/scheduler"
+	"github.com/pkg/errors"
 )
 
 const (
@@ -33,7 +34,6 @@ const (
 )
 
 var (
-
 	// allowRescheduleTransition is the transition that allows failed
 	// allocations to be force rescheduled. We create a one off
 	// variable to avoid creating a new object for every request.
@@ -58,8 +58,8 @@ func NewJobEndpoints(s *Server) *Job {
 		srv:    s,
 		logger: s.logger.Named("job"),
 		mutators: []jobMutator{
-			jobConnectHook{},
 			jobCanonicalizer{},
+			jobConnectHook{},
 			jobImpliedConstraints{},
 		},
 		validators: []jobValidator{
@@ -104,7 +104,7 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 			return structs.ErrPermissionDenied
 		}
 
-		// Validate Volume Permsissions
+		// Validate Volume Permissions
 		for _, tg := range args.Job.TaskGroups {
 			for _, vol := range tg.Volumes {
 				if vol.Type != structs.VolumeTypeHost {
@@ -215,8 +215,36 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		}
 	}
 
-	// Enforce Sentinel policies
-	policyWarnings, err := j.enforceSubmitJob(args.PolicyOverride, args.Job)
+	// helper function that checks if the "operator token" supplied with the
+	// job has sufficient ACL permissions for establishing consul connect services
+	checkOperatorToken := func(task string) error {
+		if j.srv.config.ConsulConfig.AllowsUnauthenticated() {
+			// if consul.allow_unauthenticated is enabled (which is the default)
+			// just let the Job through without checking anything.
+			return nil
+		}
+		proxiedTask := strings.TrimPrefix(task, structs.ConnectProxyPrefix+"-")
+		ctx := context.Background()
+		if err := j.srv.consulACLs.CheckSIPolicy(ctx, proxiedTask, args.Job.ConsulToken); err != nil {
+			// not much in the way of exported error types, we could parse
+			// the content, but all errors are going to be failures anyway
+			return errors.Wrap(err, "operator token denied")
+		}
+		return nil
+	}
+
+	// Enforce that the operator has necessary Consul ACL permissions
+	for _, tg := range args.Job.ConnectTasks() {
+		for _, task := range tg {
+			if err := checkOperatorToken(task); err != nil {
+				return err
+			}
+		}
+	}
+
+	// Enforce Sentinel policies. Pass a copy of the job to prevent
+	// sentinel from altering it.
+	policyWarnings, err := j.enforceSubmitJob(args.PolicyOverride, args.Job.Copy())
 	if err != nil {
 		return err
 	}
@@ -228,6 +256,9 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 	// Clear the Vault token
 	args.Job.VaultToken = ""
 
+	// Clear the Consul token
+	args.Job.ConsulToken = ""
+
 	// Check if the job has changed at all
 	if existingJob == nil || existingJob.SpecChanged(args.Job) {
 		// Set the submit time
