@@ -44,12 +44,13 @@ func rpcClient(t *testing.T, s *Server) rpc.ClientCodec {
 
 func TestRPC_forwardLeader(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.DevDisableBootstrap = true
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
 	testutil.WaitForLeader(t, s2.RPC)
@@ -83,10 +84,11 @@ func TestRPC_forwardLeader(t *testing.T) {
 
 func TestRPC_WaitForConsistentReads(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS2 := TestServer(t, func(c *Config) {
 		c.RPCHoldTimeout = 20 * time.Millisecond
 	})
-	defer s1.Shutdown()
+	defer cleanupS2()
 	testutil.WaitForLeader(t, s1.RPC)
 
 	isLeader, _ := s1.getLeader()
@@ -124,12 +126,13 @@ func TestRPC_WaitForConsistentReads(t *testing.T) {
 
 func TestRPC_forwardRegion(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.Region = "global"
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
 	testutil.WaitForLeader(t, s2.RPC)
@@ -146,6 +149,32 @@ func TestRPC_forwardRegion(t *testing.T) {
 	}
 }
 
+func TestRPC_getServer(t *testing.T) {
+	t.Parallel()
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
+		c.Region = "global"
+	})
+	defer cleanupS2()
+	TestJoin(t, s1, s2)
+	testutil.WaitForLeader(t, s1.RPC)
+	testutil.WaitForLeader(t, s2.RPC)
+
+	// Lookup by name
+	srv, err := s1.getServer("global", s2.serf.LocalMember().Name)
+	require.NoError(t, err)
+
+	require.Equal(t, srv.Name, s2.serf.LocalMember().Name)
+
+	// Lookup by id
+	srv, err = s2.getServer("global", s1.serf.LocalMember().Tags["id"])
+	require.NoError(t, err)
+
+	require.Equal(t, srv.Name, s1.serf.LocalMember().Name)
+}
+
 func TestRPC_PlaintextRPCSucceedsWhenInUpgradeMode(t *testing.T) {
 	t.Parallel()
 	assert := assert.New(t)
@@ -158,7 +187,7 @@ func TestRPC_PlaintextRPCSucceedsWhenInUpgradeMode(t *testing.T) {
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
 
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.DataDir = path.Join(dir, "node1")
 		c.TLSConfig = &config.TLSConfig{
 			EnableRPC:            true,
@@ -169,7 +198,7 @@ func TestRPC_PlaintextRPCSucceedsWhenInUpgradeMode(t *testing.T) {
 			RPCUpgradeMode:       true,
 		}
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
 	codec := rpcClient(t, s1)
 
@@ -201,7 +230,7 @@ func TestRPC_PlaintextRPCFailsWhenNotInUpgradeMode(t *testing.T) {
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
 
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.DataDir = path.Join(dir, "node1")
 		c.TLSConfig = &config.TLSConfig{
 			EnableRPC:            true,
@@ -211,7 +240,7 @@ func TestRPC_PlaintextRPCFailsWhenNotInUpgradeMode(t *testing.T) {
 			KeyFile:              fookey,
 		}
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
 	codec := rpcClient(t, s1)
 
@@ -230,12 +259,12 @@ func TestRPC_streamingRpcConn_badMethod(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
 
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.DevDisableBootstrap = true
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
 	testutil.WaitForLeader(t, s2.RPC)
@@ -257,6 +286,7 @@ func TestRPC_streamingRpcConn_badMethod(t *testing.T) {
 func TestRPC_streamingRpcConn_badMethod_TLS(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
+
 	const (
 		cafile  = "../helper/tlsutil/testdata/ca.pem"
 		foocert = "../helper/tlsutil/testdata/nomad-foo.pem"
@@ -264,7 +294,7 @@ func TestRPC_streamingRpcConn_badMethod_TLS(t *testing.T) {
 	)
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.Region = "regionFoo"
 		c.BootstrapExpect = 2
 		c.DevMode = false
@@ -279,9 +309,9 @@ func TestRPC_streamingRpcConn_badMethod_TLS(t *testing.T) {
 			KeyFile:              fookey,
 		}
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
-	s2 := TestServer(t, func(c *Config) {
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.Region = "regionFoo"
 		c.BootstrapExpect = 2
 		c.DevMode = false
@@ -296,7 +326,7 @@ func TestRPC_streamingRpcConn_badMethod_TLS(t *testing.T) {
 			KeyFile:              fookey,
 		}
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
@@ -320,23 +350,23 @@ func TestRPC_streamingRpcConn_goodMethod_Plaintext(t *testing.T) {
 	require := require.New(t)
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.Region = "regionFoo"
 		c.BootstrapExpect = 2
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node1")
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
-	s2 := TestServer(t, func(c *Config) {
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.Region = "regionFoo"
 		c.BootstrapExpect = 2
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node2")
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
@@ -372,6 +402,7 @@ func TestRPC_streamingRpcConn_goodMethod_Plaintext(t *testing.T) {
 func TestRPC_streamingRpcConn_goodMethod_TLS(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
+
 	const (
 		cafile  = "../helper/tlsutil/testdata/ca.pem"
 		foocert = "../helper/tlsutil/testdata/nomad-foo.pem"
@@ -379,7 +410,7 @@ func TestRPC_streamingRpcConn_goodMethod_TLS(t *testing.T) {
 	)
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.Region = "regionFoo"
 		c.BootstrapExpect = 2
 		c.DevMode = false
@@ -394,9 +425,9 @@ func TestRPC_streamingRpcConn_goodMethod_TLS(t *testing.T) {
 			KeyFile:              fookey,
 		}
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
-	s2 := TestServer(t, func(c *Config) {
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.Region = "regionFoo"
 		c.BootstrapExpect = 2
 		c.DevMode = false
@@ -411,7 +442,7 @@ func TestRPC_streamingRpcConn_goodMethod_TLS(t *testing.T) {
 			KeyFile:              fookey,
 		}
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
@@ -453,8 +484,9 @@ func TestRPC_streamingRpcConn_goodMethod_TLS(t *testing.T) {
 func TestRPC_handleMultiplexV2(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
-	s := TestServer(t, nil)
-	defer s.Shutdown()
+
+	s, cleanupS := TestServer(t, nil)
+	defer cleanupS()
 	testutil.WaitForLeader(t, s.RPC)
 
 	p1, p2 := net.Pipe()
@@ -515,7 +547,7 @@ func TestRPC_TLS_in_TLS(t *testing.T) {
 		fookey  = "../helper/tlsutil/testdata/nomad-foo-key.pem"
 	)
 
-	s := TestServer(t, func(c *Config) {
+	s, cleanup := TestServer(t, func(c *Config) {
 		c.TLSConfig = &config.TLSConfig{
 			EnableRPC: true,
 			CAFile:    cafile,
@@ -524,7 +556,7 @@ func TestRPC_TLS_in_TLS(t *testing.T) {
 		}
 	})
 	defer func() {
-		s.Shutdown()
+		cleanup()
 
 		//TODO Avoid panics from logging during shutdown
 		time.Sleep(1 * time.Second)
@@ -811,7 +843,7 @@ func TestRPC_Limits_OK(t *testing.T) {
 				t.Fatalf("test fixture failure: cannot assert timeout when no timeout set (0)")
 			}
 
-			s := TestServer(t, func(c *Config) {
+			s, cleanup := TestServer(t, func(c *Config) {
 				if tc.tls {
 					c.TLSConfig = &config.TLSConfig{
 						EnableRPC: true,
@@ -824,7 +856,7 @@ func TestRPC_Limits_OK(t *testing.T) {
 				c.RPCMaxConnsPerClient = tc.limit
 			})
 			defer func() {
-				s.Shutdown()
+				cleanup()
 
 				//TODO Avoid panics from logging during shutdown
 				time.Sleep(1 * time.Second)
@@ -846,12 +878,12 @@ func TestRPC_Limits_OK(t *testing.T) {
 func TestRPC_Limits_Streaming(t *testing.T) {
 	t.Parallel()
 
-	s := TestServer(t, func(c *Config) {
+	s, cleanup := TestServer(t, func(c *Config) {
 		limits := config.DefaultLimits()
 		c.RPCMaxConnsPerClient = *limits.RPCMaxConnsPerClient
 	})
 	defer func() {
-		s.Shutdown()
+		cleanup()
 
 		//TODO Avoid panics from logging during shutdown
 		time.Sleep(1 * time.Second)
