@@ -41,10 +41,12 @@ func init() {
 		evalTableSchema,
 		allocTableSchema,
 		vaultAccessorTableSchema,
+		siTokenAccessorTableSchema,
 		aclPolicyTableSchema,
 		aclTokenTableSchema,
 		autopilotConfigTableSchema,
 		schedulerConfigTableSchema,
+		clusterMetaTableSchema,
 	}...)
 }
 
@@ -66,7 +68,7 @@ func stateStoreSchema() *memdb.DBSchema {
 	return db
 }
 
-// indexTableSchema is used for
+// indexTableSchema is used for tracking the most recent index used for each table.
 func indexTableSchema() *memdb.TableSchema {
 	return &memdb.TableSchema{
 		Name: "index",
@@ -549,6 +551,44 @@ func vaultAccessorTableSchema() *memdb.TableSchema {
 	}
 }
 
+// siTokenAccessorTableSchema returns the MemDB schema for the Service Identity
+// token accessor table. This table tracks accessors for tokens created on behalf
+// of allocations with Consul connect enabled tasks that need SI tokens.
+func siTokenAccessorTableSchema() *memdb.TableSchema {
+	return &memdb.TableSchema{
+		Name: siTokenAccessorTable,
+		Indexes: map[string]*memdb.IndexSchema{
+			// The primary index is the accessor id
+			"id": {
+				Name:         "id",
+				AllowMissing: false,
+				Unique:       true,
+				Indexer: &memdb.StringFieldIndex{
+					Field: "AccessorID",
+				},
+			},
+
+			"alloc_id": {
+				Name:         "alloc_id",
+				AllowMissing: false,
+				Unique:       false,
+				Indexer: &memdb.StringFieldIndex{
+					Field: "AllocID",
+				},
+			},
+
+			"node_id": {
+				Name:         "node_id",
+				AllowMissing: false,
+				Unique:       false,
+				Indexer: &memdb.StringFieldIndex{
+					Field: "NodeID",
+				},
+			},
+		},
+	}
+}
+
 // aclPolicyTableSchema returns the MemDB schema for the policy table.
 // This table is used to store the policies which are referenced by tokens
 func aclPolicyTableSchema() *memdb.TableSchema {
@@ -601,6 +641,12 @@ func aclTokenTableSchema() *memdb.TableSchema {
 	}
 }
 
+// singletonRecord can be used to describe tables which should contain only 1 entry.
+// Example uses include storing node config or cluster metadata blobs.
+var singletonRecord = &memdb.ConditionalIndex{
+	Conditional: func(interface{}) (bool, error) { return true, nil },
+}
+
 // schedulerConfigTableSchema returns the MemDB schema for the scheduler config table.
 // This table is used to store configuration options for the scheduler
 func schedulerConfigTableSchema() *memdb.TableSchema {
@@ -611,10 +657,22 @@ func schedulerConfigTableSchema() *memdb.TableSchema {
 				Name:         "id",
 				AllowMissing: true,
 				Unique:       true,
-				// This indexer ensures that this table is a singleton
-				Indexer: &memdb.ConditionalIndex{
-					Conditional: func(obj interface{}) (bool, error) { return true, nil },
-				},
+				Indexer:      singletonRecord, // we store only 1 scheduler config
+			},
+		},
+	}
+}
+
+// clusterMetaTableSchema returns the MemDB schema for the scheduler config table.
+func clusterMetaTableSchema() *memdb.TableSchema {
+	return &memdb.TableSchema{
+		Name: "cluster_meta",
+		Indexes: map[string]*memdb.IndexSchema{
+			"id": {
+				Name:         "id",
+				AllowMissing: false,
+				Unique:       true,
+				Indexer:      singletonRecord, // we store only 1 cluster metadata
 			},
 		},
 	}
