@@ -3,7 +3,7 @@
 package state
 
 import (
-	"github.com/mitchellh/go-testing-interface"
+	testing "github.com/mitchellh/go-testing-interface"
 )
 
 func TestInitState(t testing.T, state *StateStore) {}
