@@ -57,8 +57,7 @@ type JobDiff struct {
 // diffable. If contextual diff is enabled, objects within the job will contain
 // field information even if unchanged.
 func (j *Job) Diff(other *Job, contextual bool) (*JobDiff, error) {
-	// COMPAT: Remove "Update" in 0.7.0. Update pushed down to task groups
-	// in 0.6.0
+	// See agent.ApiJobToStructJob Update is a default for TaskGroups
 	diff := &JobDiff{Type: DiffTypeNone}
 	var oldPrimitiveFlat, newPrimitiveFlat map[string]string
 	filter := []string{"ID", "Status", "StatusDescription", "Version", "Stable", "CreateIndex",
@@ -105,6 +104,17 @@ func (j *Job) Diff(other *Job, contextual bool) (*JobDiff, error) {
 		diff.Objects = append(diff.Objects, conDiff...)
 	}
 
+	// Affinities diff
+	affinitiesDiff := primitiveObjectSetDiff(
+		interfaceSlice(j.Affinities),
+		interfaceSlice(other.Affinities),
+		[]string{"str"},
+		"Affinity",
+		contextual)
+	if affinitiesDiff != nil {
+		diff.Objects = append(diff.Objects, affinitiesDiff...)
+	}
+
 	// Task groups diff
 	tgs, err := taskGroupDiffs(j.TaskGroups, other.TaskGroups, contextual)
 	if err != nil {
@@ -228,6 +238,17 @@ func (tg *TaskGroup) Diff(other *TaskGroup, contextual bool) (*TaskGroupDiff, er
 		diff.Objects = append(diff.Objects, conDiff...)
 	}
 
+	// Affinities diff
+	affinitiesDiff := primitiveObjectSetDiff(
+		interfaceSlice(tg.Affinities),
+		interfaceSlice(other.Affinities),
+		[]string{"str"},
+		"Affinity",
+		contextual)
+	if affinitiesDiff != nil {
+		diff.Objects = append(diff.Objects, affinitiesDiff...)
+	}
+
 	// Restart policy diff
 	rDiff := primitiveObjectDiff(tg.RestartPolicy, other.RestartPolicy, nil, "RestartPolicy", contextual)
 	if rDiff != nil {
@@ -252,6 +273,16 @@ func (tg *TaskGroup) Diff(other *TaskGroup, contextual bool) (*TaskGroupDiff, er
 		diff.Objects = append(diff.Objects, uDiff)
 	}
 
+	// Network Resources diff
+	if nDiffs := networkResourceDiffs(tg.Networks, other.Networks, contextual); nDiffs != nil {
+		diff.Objects = append(diff.Objects, nDiffs...)
+	}
+
+	// Services diff
+	if sDiffs := serviceDiffs(tg.Services, other.Services, contextual); sDiffs != nil {
+		diff.Objects = append(diff.Objects, sDiffs...)
+	}
+
 	// Tasks diff
 	tasks, err := taskDiffs(tg.Tasks, other.Tasks, contextual)
 	if err != nil {
@@ -387,6 +418,17 @@ func (t *Task) Diff(other *Task, contextual bool) (*TaskDiff, error) {
 		diff.Objects = append(diff.Objects, conDiff...)
 	}
 
+	// Affinities diff
+	affinitiesDiff := primitiveObjectSetDiff(
+		interfaceSlice(t.Affinities),
+		interfaceSlice(other.Affinities),
+		[]string{"str"},
+		"Affinity",
+		contextual)
+	if affinitiesDiff != nil {
+		diff.Objects = append(diff.Objects, affinitiesDiff...)
+	}
+
 	// Config diff
 	if cDiff := configDiff(t.Config, other.Config, contextual); cDiff != nil {
 		diff.Objects = append(diff.Objects, cDiff)
@@ -547,6 +589,11 @@ func serviceDiff(old, new *Service, contextual bool) *ObjectDiff {
 		diff.Objects = append(diff.Objects, cDiffs...)
 	}
 
+	// Consul Connect diffs
+	if conDiffs := connectDiffs(old.Connect, new.Connect, contextual); conDiffs != nil {
+		diff.Objects = append(diff.Objects, conDiffs)
+	}
+
 	return diff
 }
 
@@ -672,6 +719,163 @@ func checkRestartDiff(old, new *CheckRestart, contextual bool) *ObjectDiff {
 	return diff
 }
 
+// connectDiffs returns the diff of two Consul connect objects. If contextual
+// diff is enabled, all fields will be returned, even if no diff occurred.
+func connectDiffs(old, new *ConsulConnect, contextual bool) *ObjectDiff {
+	diff := &ObjectDiff{Type: DiffTypeNone, Name: "ConsulConnect"}
+	var oldPrimitiveFlat, newPrimitiveFlat map[string]string
+
+	if reflect.DeepEqual(old, new) {
+		return nil
+	} else if old == nil {
+		old = &ConsulConnect{}
+		diff.Type = DiffTypeAdded
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	} else if new == nil {
+		new = &ConsulConnect{}
+		diff.Type = DiffTypeDeleted
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+	} else {
+		diff.Type = DiffTypeEdited
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	}
+
+	// Diff the primitive fields.
+	diff.Fields = fieldDiffs(oldPrimitiveFlat, newPrimitiveFlat, contextual)
+
+	sidecarSvcDiff := connectSidecarServiceDiff(
+		old.SidecarService, new.SidecarService, contextual)
+	if sidecarSvcDiff != nil {
+		diff.Objects = append(diff.Objects, sidecarSvcDiff)
+	}
+
+	sidecarTaskDiff := sidecarTaskDiff(old.SidecarTask, new.SidecarTask, contextual)
+	if sidecarTaskDiff != nil {
+		diff.Objects = append(diff.Objects, sidecarTaskDiff)
+	}
+
+	return diff
+}
+
+// connectSidecarServiceDiff returns the diff of two ConsulSidecarService objects.
+// If contextual diff is enabled, all fields will be returned, even if no diff occurred.
+func connectSidecarServiceDiff(old, new *ConsulSidecarService, contextual bool) *ObjectDiff {
+	diff := &ObjectDiff{Type: DiffTypeNone, Name: "SidecarService"}
+	var oldPrimitiveFlat, newPrimitiveFlat map[string]string
+
+	if reflect.DeepEqual(old, new) {
+		return nil
+	} else if old == nil {
+		old = &ConsulSidecarService{}
+		diff.Type = DiffTypeAdded
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	} else if new == nil {
+		new = &ConsulSidecarService{}
+		diff.Type = DiffTypeDeleted
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+	} else {
+		diff.Type = DiffTypeEdited
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	}
+
+	// Diff the primitive fields.
+	diff.Fields = fieldDiffs(oldPrimitiveFlat, newPrimitiveFlat, contextual)
+
+	consulProxyDiff := consulProxyDiff(old.Proxy, new.Proxy, contextual)
+	if consulProxyDiff != nil {
+		diff.Objects = append(diff.Objects, consulProxyDiff)
+	}
+
+	return diff
+}
+
+// sidecarTaskDiff returns the diff of two Task objects.
+// If contextual diff is enabled, all fields will be returned, even if no diff occurred.
+func sidecarTaskDiff(old, new *SidecarTask, contextual bool) *ObjectDiff {
+	diff := &ObjectDiff{Type: DiffTypeNone, Name: "SidecarTask"}
+	var oldPrimitiveFlat, newPrimitiveFlat map[string]string
+
+	if reflect.DeepEqual(old, new) {
+		return nil
+	} else if old == nil {
+		old = &SidecarTask{}
+		diff.Type = DiffTypeAdded
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	} else if new == nil {
+		new = &SidecarTask{}
+		diff.Type = DiffTypeDeleted
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+	} else {
+		diff.Type = DiffTypeEdited
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	}
+
+	// Diff the primitive fields.
+	diff.Fields = fieldDiffs(oldPrimitiveFlat, newPrimitiveFlat, false)
+
+	// Config diff
+	if cDiff := configDiff(old.Config, new.Config, contextual); cDiff != nil {
+		diff.Objects = append(diff.Objects, cDiff)
+	}
+
+	// Resources diff
+	if rDiff := old.Resources.Diff(new.Resources, contextual); rDiff != nil {
+		diff.Objects = append(diff.Objects, rDiff)
+	}
+
+	// LogConfig diff
+	lDiff := primitiveObjectDiff(old.LogConfig, new.LogConfig, nil, "LogConfig", contextual)
+	if lDiff != nil {
+		diff.Objects = append(diff.Objects, lDiff)
+	}
+
+	return diff
+}
+
+// consulProxyDiff returns the diff of two ConsulProxy objects.
+// If contextual diff is enabled, all fields will be returned, even if no diff occurred.
+func consulProxyDiff(old, new *ConsulProxy, contextual bool) *ObjectDiff {
+	diff := &ObjectDiff{Type: DiffTypeNone, Name: "ConsulProxy"}
+	var oldPrimitiveFlat, newPrimitiveFlat map[string]string
+
+	if reflect.DeepEqual(old, new) {
+		return nil
+	} else if old == nil {
+		old = &ConsulProxy{}
+		diff.Type = DiffTypeAdded
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	} else if new == nil {
+		new = &ConsulProxy{}
+		diff.Type = DiffTypeDeleted
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+	} else {
+		diff.Type = DiffTypeEdited
+		oldPrimitiveFlat = flatmap.Flatten(old, nil, true)
+		newPrimitiveFlat = flatmap.Flatten(new, nil, true)
+	}
+
+	// Diff the primitive fields.
+	diff.Fields = fieldDiffs(oldPrimitiveFlat, newPrimitiveFlat, contextual)
+
+	consulUpstreamsDiff := primitiveObjectSetDiff(
+		interfaceSlice(old.Upstreams),
+		interfaceSlice(new.Upstreams),
+		nil, "ConsulUpstreams", contextual)
+	if consulUpstreamsDiff != nil {
+		diff.Objects = append(diff.Objects, consulUpstreamsDiff...)
+	}
+
+	// Config diff
+	if cDiff := configDiff(old.Config, new.Config, contextual); cDiff != nil {
+		diff.Objects = append(diff.Objects, cDiff)
+	}
+
+	return diff
+}
+
 // serviceCheckDiffs diffs a set of service checks. If contextual diff is
 // enabled, unchanged fields within objects nested in the tasks will be
 // returned.
@@ -807,6 +1011,11 @@ func (r *Resources) Diff(other *Resources, contextual bool) *ObjectDiff {
 		diff.Objects = append(diff.Objects, nDiffs...)
 	}
 
+	// Requested Devices diff
+	if nDiffs := requestedDevicesDiffs(r.Devices, other.Devices, contextual); nDiffs != nil {
+		diff.Objects = append(diff.Objects, nDiffs...)
+	}
+
 	return diff
 }
 
@@ -942,6 +1151,67 @@ func portDiffs(old, new []Port, dynamic bool, contextual bool) []*ObjectDiff {
 
 }
 
+// Diff returns a diff of two requested devices. If contextual diff is enabled,
+// non-changed fields will still be returned.
+func (r *RequestedDevice) Diff(other *RequestedDevice, contextual bool) *ObjectDiff {
+	diff := &ObjectDiff{Type: DiffTypeNone, Name: "Device"}
+	var oldPrimitiveFlat, newPrimitiveFlat map[string]string
+
+	if reflect.DeepEqual(r, other) {
+		return nil
+	} else if r == nil {
+		diff.Type = DiffTypeAdded
+		newPrimitiveFlat = flatmap.Flatten(other, nil, true)
+	} else if other == nil {
+		diff.Type = DiffTypeDeleted
+		oldPrimitiveFlat = flatmap.Flatten(r, nil, true)
+	} else {
+		diff.Type = DiffTypeEdited
+		oldPrimitiveFlat = flatmap.Flatten(r, nil, true)
+		newPrimitiveFlat = flatmap.Flatten(other, nil, true)
+	}
+
+	// Diff the primitive fields.
+	diff.Fields = fieldDiffs(oldPrimitiveFlat, newPrimitiveFlat, contextual)
+
+	return diff
+}
+
+// requestedDevicesDiffs diffs a set of RequestedDevices. If contextual diff is enabled,
+// non-changed fields will still be returned.
+func requestedDevicesDiffs(old, new []*RequestedDevice, contextual bool) []*ObjectDiff {
+	makeSet := func(devices []*RequestedDevice) map[string]*RequestedDevice {
+		deviceMap := make(map[string]*RequestedDevice, len(devices))
+		for _, d := range devices {
+			deviceMap[d.Name] = d
+		}
+
+		return deviceMap
+	}
+
+	oldSet := makeSet(old)
+	newSet := makeSet(new)
+
+	var diffs []*ObjectDiff
+	for k, oldV := range oldSet {
+		newV := newSet[k]
+		if diff := oldV.Diff(newV, contextual); diff != nil {
+			diffs = append(diffs, diff)
+		}
+	}
+	for k, newV := range newSet {
+		if oldV, ok := oldSet[k]; !ok {
+			if diff := oldV.Diff(newV, contextual); diff != nil {
+				diffs = append(diffs, diff)
+			}
+		}
+	}
+
+	sort.Sort(ObjectDiffs(diffs))
+	return diffs
+
+}
+
 // configDiff returns the diff of two Task Config objects. If contextual diff is
 // enabled, all fields will be returned, even if no diff occurred.
 func configDiff(old, new map[string]interface{}, contextual bool) *ObjectDiff {
