@@ -78,6 +78,10 @@ func IsErrUnknownMethod(err error) bool {
 	return err != nil && strings.Contains(err.Error(), errUnknownMethod)
 }
 
+func IsErrRPCCoded(err error) bool {
+	return err != nil && strings.HasPrefix(err.Error(), errRPCCodedErrorPrefix)
+}
+
 // NewErrUnknownAllocation returns a new error caused by the allocation being
 // unknown.
 func NewErrUnknownAllocation(allocID string) error {
