@@ -8,23 +8,10 @@ import (
 )
 
 const (
-	// serviceJobAntiAffinityPenalty is the penalty applied
-	// to the score for placing an alloc on a node that
-	// already has an alloc for this job.
-	serviceJobAntiAffinityPenalty = 20.0
-
-	// batchJobAntiAffinityPenalty is the same as the
-	// serviceJobAntiAffinityPenalty but for batch type jobs.
-	batchJobAntiAffinityPenalty = 10.0
-
-	// previousFailedAllocNodePenalty is a scoring penalty for nodes
-	// that a failed allocation was previously run on
-	previousFailedAllocNodePenalty = 50.0
-
 	// skipScoreThreshold is a threshold used in the limit iterator to skip nodes
-	// that have a score lower than this. -10 is the highest possible score for a
-	// node with penalty (based on batchJobAntiAffinityPenalty)
-	skipScoreThreshold = -10.0
+	// that have a score lower than this. -1 is the lowest possible score for a
+	// node with penalties (based on job anti affinity and node rescheduling penalties
+	skipScoreThreshold = 0.0
 
 	// maxSkip limits the number of nodes that can be skipped in the limit iterator
 	maxSkip = 3
@@ -41,12 +28,13 @@ type Stack interface {
 	SetJob(job *structs.Job)
 
 	// Select is used to select a node for the task group
-	Select(tg *structs.TaskGroup, options *SelectOptions) (*RankedNode, *structs.Resources)
+	Select(tg *structs.TaskGroup, options *SelectOptions) *RankedNode
 }
 
 type SelectOptions struct {
 	PenaltyNodeIDs map[string]struct{}
 	PreferredNodes []*structs.Node
+	Preempt        bool
 }
 
 // GenericStack is the Stack used for the Generic scheduler. It is
@@ -56,87 +44,24 @@ type GenericStack struct {
 	ctx    Context
 	source *StaticIterator
 
-	wrappedChecks       *FeasibilityWrapper
-	quota               FeasibleIterator
-	jobConstraint       *ConstraintChecker
-	taskGroupDrivers    *DriverChecker
-	taskGroupConstraint *ConstraintChecker
+	wrappedChecks        *FeasibilityWrapper
+	quota                FeasibleIterator
+	jobConstraint        *ConstraintChecker
+	taskGroupDrivers     *DriverChecker
+	taskGroupConstraint  *ConstraintChecker
+	taskGroupDevices     *DeviceChecker
+	taskGroupHostVolumes *HostVolumeChecker
 
 	distinctHostsConstraint    *DistinctHostsIterator
 	distinctPropertyConstraint *DistinctPropertyIterator
 	binPack                    *BinPackIterator
 	jobAntiAff                 *JobAntiAffinityIterator
-	nodeAntiAff                *NodeAntiAffinityIterator
+	nodeReschedulingPenalty    *NodeReschedulingPenaltyIterator
 	limit                      *LimitIterator
 	maxScore                   *MaxScoreIterator
-}
-
-// NewGenericStack constructs a stack used for selecting service placements
-func NewGenericStack(batch bool, ctx Context) *GenericStack {
-	// Create a new stack
-	s := &GenericStack{
-		batch: batch,
-		ctx:   ctx,
-	}
-
-	// Create the source iterator. We randomize the order we visit nodes
-	// to reduce collisions between schedulers and to do a basic load
-	// balancing across eligible nodes.
-	s.source = NewRandomIterator(ctx, nil)
-
-	// Create the quota iterator to determine if placements would result in the
-	// quota attached to the namespace of the job to go over.
-	s.quota = NewQuotaIterator(ctx, s.source)
-
-	// Attach the job constraints. The job is filled in later.
-	s.jobConstraint = NewConstraintChecker(ctx, nil)
-
-	// Filter on task group drivers first as they are faster
-	s.taskGroupDrivers = NewDriverChecker(ctx, nil)
-
-	// Filter on task group constraints second
-	s.taskGroupConstraint = NewConstraintChecker(ctx, nil)
-
-	// Create the feasibility wrapper which wraps all feasibility checks in
-	// which feasibility checking can be skipped if the computed node class has
-	// previously been marked as eligible or ineligible. Generally this will be
-	// checks that only needs to examine the single node to determine feasibility.
-	jobs := []FeasibilityChecker{s.jobConstraint}
-	tgs := []FeasibilityChecker{s.taskGroupDrivers, s.taskGroupConstraint}
-	s.wrappedChecks = NewFeasibilityWrapper(ctx, s.quota, jobs, tgs)
-
-	// Filter on distinct host constraints.
-	s.distinctHostsConstraint = NewDistinctHostsIterator(ctx, s.wrappedChecks)
-
-	// Filter on distinct property constraints.
-	s.distinctPropertyConstraint = NewDistinctPropertyIterator(ctx, s.distinctHostsConstraint)
-
-	// Upgrade from feasible to rank iterator
-	rankSource := NewFeasibleRankIterator(ctx, s.distinctPropertyConstraint)
-
-	// Apply the bin packing, this depends on the resources needed
-	// by a particular task group. Only enable eviction for the service
-	// scheduler as that logic is expensive.
-	evict := !batch
-	s.binPack = NewBinPackIterator(ctx, rankSource, evict, 0)
-
-	// Apply the job anti-affinity iterator. This is to avoid placing
-	// multiple allocations on the same node for this job. The penalty
-	// is less for batch jobs as it matters less.
-	penalty := serviceJobAntiAffinityPenalty
-	if batch {
-		penalty = batchJobAntiAffinityPenalty
-	}
-	s.jobAntiAff = NewJobAntiAffinityIterator(ctx, s.binPack, penalty, "")
-
-	s.nodeAntiAff = NewNodeAntiAffinityIterator(ctx, s.jobAntiAff, previousFailedAllocNodePenalty)
-
-	// Apply a limit function. This is to avoid scanning *every* possible node.
-	s.limit = NewLimitIterator(ctx, s.nodeAntiAff, 2, skipScoreThreshold, maxSkip)
-
-	// Select the node with the maximum score for placement
-	s.maxScore = NewMaxScoreIterator(ctx, s.limit)
-	return s
+	nodeAffinity               *NodeAffinityIterator
+	spread                     *SpreadIterator
+	scoreNorm                  *ScoreNormalizationIterator
 }
 
 func (s *GenericStack) SetNodes(baseNodes []*structs.Node) {
@@ -165,8 +90,10 @@ func (s *GenericStack) SetJob(job *structs.Job) {
 	s.jobConstraint.SetConstraints(job.Constraints)
 	s.distinctHostsConstraint.SetJob(job)
 	s.distinctPropertyConstraint.SetJob(job)
-	s.binPack.SetPriority(job.Priority)
-	s.jobAntiAff.SetJob(job.ID)
+	s.binPack.SetJob(job)
+	s.jobAntiAff.SetJob(job)
+	s.nodeAffinity.SetJob(job)
+	s.spread.SetJob(job)
 	s.ctx.Eligibility().SetJob(job)
 
 	if contextual, ok := s.quota.(ContextualIterator); ok {
@@ -174,7 +101,7 @@ func (s *GenericStack) SetJob(job *structs.Job) {
 	}
 }
 
-func (s *GenericStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*RankedNode, *structs.Resources) {
+func (s *GenericStack) Select(tg *structs.TaskGroup, options *SelectOptions) *RankedNode {
 
 	// This block handles trying to select from preferred nodes if options specify them
 	// It also sets back the set of nodes to the original nodes
@@ -183,9 +110,9 @@ func (s *GenericStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*R
 		s.source.SetNodes(options.PreferredNodes)
 		optionsNew := *options
 		optionsNew.PreferredNodes = nil
-		if option, resources := s.Select(tg, &optionsNew); option != nil {
+		if option := s.Select(tg, &optionsNew); option != nil {
 			s.source.SetNodes(originalNodes)
-			return option, resources
+			return option
 		}
 		s.source.SetNodes(originalNodes)
 		return s.Select(tg, &optionsNew)
@@ -202,12 +129,24 @@ func (s *GenericStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*R
 	// Update the parameters of iterators
 	s.taskGroupDrivers.SetDrivers(tgConstr.drivers)
 	s.taskGroupConstraint.SetConstraints(tgConstr.constraints)
+	s.taskGroupDevices.SetTaskGroup(tg)
+	s.taskGroupHostVolumes.SetVolumes(tg.Volumes)
 	s.distinctHostsConstraint.SetTaskGroup(tg)
 	s.distinctPropertyConstraint.SetTaskGroup(tg)
 	s.wrappedChecks.SetTaskGroup(tg.Name)
 	s.binPack.SetTaskGroup(tg)
 	if options != nil {
-		s.nodeAntiAff.SetPenaltyNodes(options.PenaltyNodeIDs)
+		s.binPack.evict = options.Preempt
+	}
+	s.jobAntiAff.SetTaskGroup(tg)
+	if options != nil {
+		s.nodeReschedulingPenalty.SetPenaltyNodes(options.PenaltyNodeIDs)
+	}
+	s.nodeAffinity.SetTaskGroup(tg)
+	s.spread.SetTaskGroup(tg)
+
+	if s.nodeAffinity.hasAffinities() || s.spread.hasSpreads() {
+		s.limit.SetLimit(math.MaxInt32)
 	}
 
 	if contextual, ok := s.quota.(ContextualIterator); ok {
@@ -217,30 +156,28 @@ func (s *GenericStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*R
 	// Find the node with the max score
 	option := s.maxScore.Next()
 
-	// Ensure that the task resources were specified
-	if option != nil && len(option.TaskResources) != len(tg.Tasks) {
-		for _, task := range tg.Tasks {
-			option.SetTaskResources(task, task.Resources)
-		}
-	}
-
 	// Store the compute time
 	s.ctx.Metrics().AllocationTime = time.Since(start)
-	return option, tgConstr.size
+	return option
 }
 
 // SystemStack is the Stack used for the System scheduler. It is designed to
 // attempt to make placements on all nodes.
 type SystemStack struct {
-	ctx                        Context
-	source                     *StaticIterator
-	wrappedChecks              *FeasibilityWrapper
-	quota                      FeasibleIterator
-	jobConstraint              *ConstraintChecker
-	taskGroupDrivers           *DriverChecker
-	taskGroupConstraint        *ConstraintChecker
+	ctx    Context
+	source *StaticIterator
+
+	wrappedChecks        *FeasibilityWrapper
+	quota                FeasibleIterator
+	jobConstraint        *ConstraintChecker
+	taskGroupDrivers     *DriverChecker
+	taskGroupConstraint  *ConstraintChecker
+	taskGroupDevices     *DeviceChecker
+	taskGroupHostVolumes *HostVolumeChecker
+
 	distinctPropertyConstraint *DistinctPropertyIterator
 	binPack                    *BinPackIterator
+	scoreNorm                  *ScoreNormalizationIterator
 }
 
 // NewSystemStack constructs a stack used for selecting service placements
@@ -265,12 +202,18 @@ func NewSystemStack(ctx Context) *SystemStack {
 	// Filter on task group constraints second
 	s.taskGroupConstraint = NewConstraintChecker(ctx, nil)
 
+	// Filter on task group host volumes
+	s.taskGroupHostVolumes = NewHostVolumeChecker(ctx)
+
+	// Filter on task group devices
+	s.taskGroupDevices = NewDeviceChecker(ctx)
+
 	// Create the feasibility wrapper which wraps all feasibility checks in
 	// which feasibility checking can be skipped if the computed node class has
 	// previously been marked as eligible or ineligible. Generally this will be
 	// checks that only needs to examine the single node to determine feasibility.
 	jobs := []FeasibilityChecker{s.jobConstraint}
-	tgs := []FeasibilityChecker{s.taskGroupDrivers, s.taskGroupConstraint}
+	tgs := []FeasibilityChecker{s.taskGroupDrivers, s.taskGroupConstraint, s.taskGroupHostVolumes, s.taskGroupDevices}
 	s.wrappedChecks = NewFeasibilityWrapper(ctx, s.quota, jobs, tgs)
 
 	// Filter on distinct property constraints.
@@ -282,7 +225,15 @@ func NewSystemStack(ctx Context) *SystemStack {
 	// Apply the bin packing, this depends on the resources needed
 	// by a particular task group. Enable eviction as system jobs are high
 	// priority.
-	s.binPack = NewBinPackIterator(ctx, rankSource, true, 0)
+	_, schedConfig, _ := s.ctx.State().SchedulerConfig()
+	enablePreemption := true
+	if schedConfig != nil {
+		enablePreemption = schedConfig.PreemptionConfig.SystemSchedulerEnabled
+	}
+	s.binPack = NewBinPackIterator(ctx, rankSource, enablePreemption, 0)
+
+	// Apply score normalization
+	s.scoreNorm = NewScoreNormalizationIterator(ctx, s.binPack)
 	return s
 }
 
@@ -294,7 +245,7 @@ func (s *SystemStack) SetNodes(baseNodes []*structs.Node) {
 func (s *SystemStack) SetJob(job *structs.Job) {
 	s.jobConstraint.SetConstraints(job.Constraints)
 	s.distinctPropertyConstraint.SetJob(job)
-	s.binPack.SetPriority(job.Priority)
+	s.binPack.SetJob(job)
 	s.ctx.Eligibility().SetJob(job)
 
 	if contextual, ok := s.quota.(ContextualIterator); ok {
@@ -302,9 +253,9 @@ func (s *SystemStack) SetJob(job *structs.Job) {
 	}
 }
 
-func (s *SystemStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*RankedNode, *structs.Resources) {
+func (s *SystemStack) Select(tg *structs.TaskGroup, options *SelectOptions) *RankedNode {
 	// Reset the binpack selector and context
-	s.binPack.Reset()
+	s.scoreNorm.Reset()
 	s.ctx.Reset()
 	start := time.Now()
 
@@ -314,6 +265,8 @@ func (s *SystemStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*Ra
 	// Update the parameters of iterators
 	s.taskGroupDrivers.SetDrivers(tgConstr.drivers)
 	s.taskGroupConstraint.SetConstraints(tgConstr.constraints)
+	s.taskGroupDevices.SetTaskGroup(tg)
+	s.taskGroupHostVolumes.SetVolumes(tg.Volumes)
 	s.wrappedChecks.SetTaskGroup(tg.Name)
 	s.distinctPropertyConstraint.SetTaskGroup(tg)
 	s.binPack.SetTaskGroup(tg)
@@ -323,16 +276,9 @@ func (s *SystemStack) Select(tg *structs.TaskGroup, options *SelectOptions) (*Ra
 	}
 
 	// Get the next option that satisfies the constraints.
-	option := s.binPack.Next()
-
-	// Ensure that the task resources were specified
-	if option != nil && len(option.TaskResources) != len(tg.Tasks) {
-		for _, task := range tg.Tasks {
-			option.SetTaskResources(task, task.Resources)
-		}
-	}
+	option := s.scoreNorm.Next()
 
 	// Store the compute time
 	s.ctx.Metrics().AllocationTime = time.Since(start)
-	return option, tgConstr.size
+	return option
 }
