@@ -4,6 +4,7 @@ import { computed } from '@ember/object';
 
 export default Component.extend({
   system: service(),
+  router: service(),
 
   sortedNamespaces: computed('system.namespaces.@each.name', function() {
     const namespaces = this.get('system.namespaces').toArray() || [];
@@ -31,5 +32,13 @@ export default Component.extend({
     });
   }),
 
-  onNamespaceChange() {},
+  onHamburgerClick() {},
+
+  gotoJobsForNamespace(namespace) {
+    if (!namespace || !namespace.get('id')) return;
+
+    this.router.transitionTo('jobs', {
+      queryParams: { namespace: namespace.get('id') },
+    });
+  },
 });
