@@ -1,21 +1,22 @@
 import Component from '@ember/component';
 import { computed } from '@ember/object';
+import { computed as overridable } from 'ember-overridable-computed';
 import moment from 'moment';
 
 export default Component.extend({
   tagName: 'ol',
   classNames: ['timeline'],
 
-  deployments: computed(() => []),
+  deployments: overridable(() => []),
 
   sortedDeployments: computed('deployments.@each.versionSubmitTime', function() {
-    return this.get('deployments')
+    return this.deployments
       .sortBy('versionSubmitTime')
       .reverse();
   }),
 
   annotatedDeployments: computed('sortedDeployments.@each.version', function() {
-    const deployments = this.get('sortedDeployments');
+    const deployments = this.sortedDeployments;
     return deployments.map((deployment, index) => {
       const meta = {};
 
