@@ -1,5 +1,5 @@
 import Component from '@ember/component';
-import { computed } from '@ember/object';
+import { computed as overridable } from 'ember-overridable-computed';
 import { inject as service } from '@ember/service';
 
 export default Component.extend({
@@ -10,8 +10,8 @@ export default Component.extend({
   allocationId: null,
 
   // An allocation can also be provided directly
-  allocation: computed('allocationId', function() {
-    return this.get('store').findRecord('allocation', this.get('allocationId'));
+  allocation: overridable('allocationId', function() {
+    return this.store.findRecord('allocation', this.allocationId);
   }),
 
   time: null,
