@@ -12,16 +12,26 @@ export default Component.extend({
   // Used to throttle sets to searchTerm
   debounce: 150,
 
+  // A hook that's called when the search value changes
+  onChange() {},
+
   classNames: ['search-box', 'field', 'has-addons'],
 
   actions: {
     setSearchTerm(e) {
       this.set('_searchTerm', e.target.value);
-      run.debounce(this, updateSearch, this.get('debounce'));
+      run.debounce(this, updateSearch, this.debounce);
+    },
+
+    clear() {
+      this.set('_searchTerm', '');
+      run.debounce(this, updateSearch, this.debounce);
     },
   },
 });
 
 function updateSearch() {
-  this.set('searchTerm', this.get('_searchTerm'));
+  const newTerm = this._searchTerm;
+  this.onChange(newTerm);
+  this.set('searchTerm', newTerm);
 }
