@@ -1,12 +1,10 @@
-import Controller, { inject as controller } from '@ember/controller';
+import Controller from '@ember/controller';
 import { computed } from '@ember/object';
+import { computed as overridable } from 'ember-overridable-computed';
 import { alias } from '@ember/object/computed';
+import { task } from 'ember-concurrency';
 
 export default Controller.extend({
-  taskController: controller('allocations.allocation.task'),
-
-  breadcrumbs: alias('taskController.breadcrumbs'),
-
   network: alias('model.resources.networks.firstObject'),
   ports: computed('network.reservedPorts.[]', 'network.dynamicPorts.[]', function() {
     return (this.get('network.reservedPorts') || [])
@@ -24,4 +22,24 @@ export default Controller.extend({
       )
       .sortBy('name');
   }),
+
+  error: overridable(() => {
+    // { title, description }
+    return null;
+  }),
+
+  onDismiss() {
+    this.set('error', null);
+  },
+
+  restartTask: task(function*() {
+    try {
+      yield this.model.restart();
+    } catch (err) {
+      this.set('error', {
+        title: 'Could Not Restart Task',
+        description: 'Your ACL token does not grant allocation lifecycle permissions.',
+      });
+    }
+  }),
 });
