@@ -1,13 +1,11 @@
 import { inject as service } from '@ember/service';
 import { alias } from '@ember/object/computed';
-import Controller, { inject as controller } from '@ember/controller';
+import Controller from '@ember/controller';
 import WithNamespaceResetting from 'nomad-ui/mixins/with-namespace-resetting';
 
 export default Controller.extend(WithNamespaceResetting, {
   system: service(),
 
-  jobController: controller('jobs.job'),
-
   queryParams: {
     currentPage: 'page',
     sortProperty: 'sort',
@@ -16,12 +14,11 @@ export default Controller.extend(WithNamespaceResetting, {
 
   currentPage: 1,
 
+  job: alias('model'),
+
   sortProperty: 'name',
   sortDescending: false,
 
-  breadcrumbs: alias('jobController.breadcrumbs'),
-  job: alias('model'),
-
   actions: {
     gotoTaskGroup(taskGroup) {
       this.transitionToRoute('jobs.job.task-group', taskGroup.get('job'), taskGroup);
