@@ -1,14 +1,11 @@
 import { alias } from '@ember/object/computed';
-import Controller, { inject as controller } from '@ember/controller';
+import Controller from '@ember/controller';
 import { computed } from '@ember/object';
 import Sortable from 'nomad-ui/mixins/sortable';
 import Searchable from 'nomad-ui/mixins/searchable';
 import WithNamespaceResetting from 'nomad-ui/mixins/with-namespace-resetting';
-import { qpBuilder } from 'nomad-ui/utils/classes/query-params';
 
 export default Controller.extend(Sortable, Searchable, WithNamespaceResetting, {
-  jobController: controller('jobs.job'),
-
   queryParams: {
     currentPage: 'page',
     searchTerm: 'search',
@@ -32,19 +29,6 @@ export default Controller.extend(Sortable, Searchable, WithNamespaceResetting, {
   listToSearch: alias('listSorted'),
   sortedAllocations: alias('listSearched'),
 
-  breadcrumbs: computed('jobController.breadcrumbs.[]', 'model.{name}', function() {
-    return this.get('jobController.breadcrumbs').concat([
-      {
-        label: this.get('model.name'),
-        args: [
-          'jobs.job.task-group',
-          this.get('model.name'),
-          qpBuilder({ jobNamespace: this.get('model.job.namespace.name') || 'default' }),
-        ],
-      },
-    ]);
-  }),
-
   actions: {
     gotoAllocation(allocation) {
       this.transitionToRoute('allocations.allocation', allocation);
