@@ -1,8 +1,8 @@
 import Helper from '@ember/component/helper';
 import formatDuration from '../utils/format-duration';
 
-function formatDurationHelper([duration], { units }) {
-  return formatDuration(duration, units);
+function formatDurationHelper([duration], { units, longForm }) {
+  return formatDuration(duration, units, longForm);
 }
 
 export default Helper.helper(formatDurationHelper);
