@@ -17,12 +17,13 @@ export default Model.extend({
   hasPlacementFailures: bool('failedTGAllocs.length'),
   isBlocked: equal('status', 'blocked'),
 
-  // TEMPORARY: https://github.com/emberjs/data/issues/5209
-  originalJobId: attr('string'),
-
   job: belongsTo('job'),
 
   modifyIndex: attr('number'),
+  modifyTime: attr('date'),
+
+  createIndex: attr('number'),
+  createTime: attr('date'),
 
   waitUntil: attr('date'),
 });
