@@ -2,6 +2,6 @@ import ApplicationSerializer from './application';
 
 export default ApplicationSerializer.extend({
   normalize(typeHash, hash) {
-    return this._super(typeHash, { Attributes: hash });
+    return this._super(typeHash, { NodeAttributes: hash });
   },
 });
