@@ -1,14 +1,15 @@
 import { assert } from '@ember/debug';
 import Mixin from '@ember/object/mixin';
 import { computed } from '@ember/object';
+import { computed as overridable } from 'ember-overridable-computed';
 import { assign } from '@ember/polyfills';
-import queryString from 'npm:query-string';
+import queryString from 'query-string';
 
 const MAX_OUTPUT_LENGTH = 50000;
 
 export default Mixin.create({
   url: '',
-  params: computed(() => ({})),
+  params: overridable(() => ({})),
   logFetch() {
     assert('Loggers need a logFetch method, which should have an interface like window.fetch');
   },
@@ -16,18 +17,18 @@ export default Mixin.create({
   endOffset: null,
 
   offsetParams: computed('endOffset', function() {
-    const endOffset = this.get('endOffset');
+    const endOffset = this.endOffset;
     return endOffset
       ? { origin: 'start', offset: endOffset }
       : { origin: 'end', offset: MAX_OUTPUT_LENGTH };
   }),
 
-  additionalParams: computed(() => ({})),
+  additionalParams: overridable(() => ({})),
 
   fullUrl: computed('url', 'params', 'offsetParams', 'additionalParams', function() {
     const queryParams = queryString.stringify(
-      assign({}, this.get('params'), this.get('offsetParams'), this.get('additionalParams'))
+      assign({}, this.params, this.offsetParams, this.additionalParams)
     );
-    return `${this.get('url')}?${queryParams}`;
+    return `${this.url}?${queryParams}`;
   }),
 });
