@@ -28,11 +28,17 @@ export default EmberObject.extend(AbstractLogger, {
     const url = this.fullUrl;
     const logFetch = this.logFetch;
 
-    let streamClosed = false;
-    let buffer = '';
-
-    const decoder = new TextDecoder();
-    const reader = yield logFetch(url).then(res => res.body.getReader(), fetchFailure(url));
+    const reader = yield logFetch(url).then(res => {
+      const reader = res.body.getReader();
+      // It's possible that the logger was stopped between the time
+      // polling was started and the log request responded.
+      // If the logger was stopped, the reader needs to be immediately
+      // canceled to prevent an endless request running in the background.
+      if (this.poll.isRunning) {
+        return reader;
+      }
+      reader.cancel();
+    }, fetchFailure(url));
 
     if (!reader) {
       return;
@@ -40,6 +46,10 @@ export default EmberObject.extend(AbstractLogger, {
 
     this.set('reader', reader);
 
+    let streamClosed = false;
+    let buffer = '';
+    const decoder = new TextDecoder();
+
     while (!streamClosed) {
       yield reader.read().then(({ value, done }) => {
         streamClosed = done;
