@@ -3,6 +3,7 @@ const EmberApp = require('ember-cli/lib/broccoli/ember-app');
 
 const environment = EmberApp.env();
 const isProd = environment === 'production';
+const isTest = environment === 'test';
 
 module.exports = function(defaults) {
   var app = new EmberApp(defaults, {
@@ -10,7 +11,31 @@ module.exports = function(defaults) {
       blacklist: isProd ? ['ember-freestyle'] : [],
     },
     svg: {
-      paths: ['public/images/icons'],
+      paths: ['node_modules/@hashicorp/structure-icons/dist', 'public/images/icons'],
+      optimize: {
+        plugins: [{ removeViewBox: false }],
+      },
+    },
+    codemirror: {
+      modes: ['javascript'],
+    },
+    funnel: {
+      enabled: isProd,
+      exclude: [
+        `${defaults.project.pkg.name}/components/freestyle/**/*`,
+        `${defaults.project.pkg.name}/templates/components/freestyle/**/*`,
+      ],
+    },
+    babel: {
+      plugins: ['@babel/plugin-proposal-object-rest-spread'],
+    },
+    'ember-cli-babel': {
+      includePolyfill: isProd,
+    },
+    hinting: isTest,
+    tests: isTest,
+    sourcemaps: {
+      enabled: false,
     },
   });
 
