@@ -1,4 +1,5 @@
-import { Factory, faker } from 'ember-cli-mirage';
+import { Factory } from 'ember-cli-mirage';
+import faker from 'nomad-ui/mirage/faker';
 import { provide } from '../utils';
 import { DATACENTERS } from '../common';
 
@@ -9,8 +10,8 @@ export default Factory.extend({
   id: i => (i / 100 >= 1 ? `${UUIDS[i]}-${i}` : UUIDS[i]),
   name: () => `nomad@${faker.internet.ip()}`,
 
-  status: faker.list.random(...AGENT_STATUSES),
-  serf_port: faker.random.number({ min: 4000, max: 4999 }),
+  status: () => faker.helpers.randomize(AGENT_STATUSES),
+  serf_port: () => faker.random.number({ min: 4000, max: 4999 }),
 
   address() {
     return this.name.split('@')[1];
@@ -20,7 +21,7 @@ export default Factory.extend({
     const rpcPortCandidate = faker.random.number({ min: 4000, max: 4999 });
     return {
       port: rpcPortCandidate === this.serf_port ? rpcPortCandidate + 1 : rpcPortCandidate,
-      dc: faker.list.random(...DATACENTERS)(),
+      dc: faker.helpers.randomize(DATACENTERS),
     };
   },
 });
