@@ -1,4 +1,6 @@
-import { Factory, faker, trait } from 'ember-cli-mirage';
+import { Factory, trait } from 'ember-cli-mirage';
+
+import faker from 'nomad-ui/mirage/faker';
 
 export default Factory.extend({
   // Hidden property used to compute the Summary hash
