@@ -1,4 +1,5 @@
-import { Factory, faker } from 'ember-cli-mirage';
+import { Factory } from 'ember-cli-mirage';
+import faker from 'nomad-ui/mirage/faker';
 import { generateResources } from '../common';
 
 const DRIVERS = ['docker', 'java', 'rkt', 'qemu', 'exec', 'raw_exec'];
@@ -9,8 +10,8 @@ export default Factory.extend({
 
   JobID: '',
 
-  name: id => `task-${faker.hacker.noun()}-${id}`,
-  driver: faker.list.random(...DRIVERS),
+  name: id => `task-${faker.hacker.noun().dasherize()}-${id}`,
+  driver: () => faker.helpers.randomize(DRIVERS),
 
   Resources: generateResources,
 });
