@@ -10,14 +10,16 @@ module.exports = {
     //   ci: ['--headless', '--disable-gpu', '--remote-debugging-port=9222', '--window-size=1440,900'],
     // },
     Chrome: {
-      mode: 'ci',
-      args: [
+      ci: [
+        // --no-sandbox is needed when running Chrome inside a container
+        process.env.CI ? '--no-sandbox' : null,
         '--headless',
-        '--no-sandbox',
-        '--disable-gpu',
-        '--remote-debugging-port=9222',
-        '--window-size=1440,900',
-      ],
-    },
-  },
+        '--disable-dev-shm-usage',
+        '--disable-software-rasterizer',
+        '--mute-audio',
+        '--remote-debugging-port=0',
+        '--window-size=1440,900'
+      ].filter(Boolean)
+    }
+  }
 };
